# import requests
# import json
# from ....settings import (
#     COMMUNICATION_SERVICE_BASE_URL,
#     NOTICE_SERVICE_BASE_URL,
#     REDIS,
#     QUEUE_SERVICE_BASE_URL,
# )
# import logging
# import os
# import redis
# from ....utils import get_s3_client
# import shutil
# from botocore.exceptions import ClientError
# from xlsxwriter import Workbook
# import re
# from cg_kafka.producer.JSONProducer import JSONProducer
# from ....settings import MERGE_NOTICE_CLIENT_ID, MERGE_NOTICE_TOPIC, NUMBER_OF_PARTITIONS_FOR_MERGE_NOTICE_TOPIC
# from ....choices import EXCEL_WORKSHEET_HYPERLINK_LIMIT, MERGE_FILE_NAME_REPLACE_CHARACTER

# logger = logging.getLogger(__name__)
# logger.setLevel(logging.DEBUG)

# redis_conf = REDIS
# redis_obj = redis.Redis(host=redis_conf["HOST"], port=redis_conf["PORT"])

# s3 = get_s3_client()

# numbers = re.compile(r"(\d+)")
# non_numbers = re.compile(r"(\D+)")


# def file_system_Sort(value):
#     parts = non_numbers.split(value)
#     parts[1::2] = list(map(lambda y: str.lower(y), parts[1::2]))
#     parts[1::2] = map(str, parts[1::2])
#     part = "".join(parts)
#     parts = numbers.split(part)
#     parts[1::2] = map(int, parts[1::2])
#     return parts


# def generate_notices_zip(local_pdf_directory_path, local_zip_file_path):
#     logger.info("notice_callback.commons.generate_notices")
#     if os.path.exists(local_pdf_directory_path):
#         dir_content = os.listdir(local_pdf_directory_path)
#         if len(dir_content) > 0:
#             shutil.make_archive(local_zip_file_path, "zip", local_pdf_directory_path)
#             return True
#     return False


# def upload_file_to_s3(s3, local_file_name, bucket, s3_object_name):
#     logger.info("notice_callback.commons.upload_file_to_s3")
#     try:
#         response = s3.upload_file(local_file_name, bucket, s3_object_name, ExtraArgs={"ACL": "public-read"})
#     except ClientError as e:
#         logger.error(f"notice_callback.commons.upload_file_to_s3.exception : {str(e)}")
#         return False

#     return True


# def changeCamelCaseToSnakeCase(str):
#     str = "".join("_" + i.lower() if i.isupper() else i for i in str).lstrip("_")
#     str = "".join("_" if i == " " else i for i in str)
#     return str


# def generate_tracking_excel(**kwargs):
#     logger.info("notice_callback.commons.generate_tracking_excel")
#     file_path = kwargs["file_path"]
#     notices = kwargs["notice_data"]
#     is_linked_loan = kwargs.get("is_linked_loan", False)

#     workbook = Workbook(filename=file_path)
#     worksheet = workbook.add_worksheet()
#     links_written_in_worksheet = 0

#     row = 0
#     col = 0

#     worksheet.write_string(row, col, "system_id")
#     worksheet.write_string(row, col + 1, "loan_id")
#     worksheet.write_string(row, col + 2, "file_name")
#     worksheet.write_string(row, col + 3, "tracking_no")
#     worksheet.write_string(row, col + 4, "allocation_month")
#     worksheet.write_string(row, col + 5, "notice_type")
#     worksheet.write_string(row, col + 6, "notice_id")
#     worksheet.write_string(row, col + 7, "notice_link")
#     worksheet.write_string(row, col + 8, "batch_id")

#     if is_linked_loan:
#         worksheet.write_string(0, 9, "linked_loan_id")

#     address_headers_set = set()
#     for notice in notices:
#         if notice["primary_address"]:
#             for key in json.loads(notice["primary_address"]):
#                 address_headers_set.add(key)

#     start_index = 10 if is_linked_loan else 9

#     for i, key in enumerate(address_headers_set):
#         worksheet.write_string(row, col + i + start_index, changeCamelCaseToSnakeCase(key))

#     row += 1

#     for notice in notices:
#         worksheet.write_string(row, col, str(notice["id"]))
#         worksheet.write_string(row, col + 1, "'" + str(notice["loan_id"]))
#         worksheet.write_string(row, col + 2, notice["local_pdf_file_name"])
#         worksheet.write_string(row, col + 3, str(notice["tracking_id"] or ""))
#         worksheet.write_string(row, col + 4, str(notice["allocation_month"] or ""))
#         worksheet.write_string(row, col + 5, str(notice["notice_type"] or ""))
#         worksheet.write_string(row, col + 6, notice["notice_id"])
#         if links_written_in_worksheet < EXCEL_WORKSHEET_HYPERLINK_LIMIT:
#             if str(notice["s3_link"]):
#                 worksheet.write_url(row, col + 7, str(notice["s3_link"] or ""))
#                 links_written_in_worksheet += 1
#         else:
#             worksheet.write_string(row, col + 7, str(notice["s3_link"] or ""))

#         worksheet.write_string(row, col + 8, kwargs["batch_id"])

#         if is_linked_loan:
#             worksheet.write_string(row, col + 9, "'" + str(notice["linked_loan_id"] or ""))

#         if notice["primary_address"]:
#             address = json.loads(notice["primary_address"])
#             for i, key in enumerate(address_headers_set):
#                 worksheet.write_string(row, col + i + start_index, str(address.get(key, "")))
#         row += 1

#     for i, key in enumerate(address_headers_set):
#         try:
#             if "cell_1" in key:
#                 key = key.split("table_1_cell_1_")[1]
#             elif "table_1_" in key:
#                 key = key.split("table_1_")[1]
#             key = key.split("_")[0] if "address" in key else key.replace("_", " ")
#         except Exception as e:
#             logger.error(f"key spilitting error: {str(e)}, {key}")
#         worksheet.write_string(0, 0 + i + start_index, changeCamelCaseToSnakeCase(key))

#     workbook.close()


# def generate_tracking_excel_with_merged_pdf_file_mapping(**kwargs):
#     logger.info("notice_callback.commons.generate_tracking_excel_with_merged_pdf_file_mapping")
#     file_path = kwargs["local_tracking_excel_file_with_merge_file_name_mapping_path"]
#     notices = kwargs["notice_data"]
#     is_linked_loan = kwargs.get("is_linked_loan", False)
#     is_merged_lot = True if kwargs.get("merged_lot_size", None) else False
#     notice_id_merged_file_name_mapping = kwargs.get("notice_id_merged_file_name_mapping", {})

#     workbook = Workbook(filename=file_path)
#     worksheet = workbook.add_worksheet()
#     links_written_in_worksheet = 0

#     row = 0
#     col = 0

#     worksheet.write_string(row, col, "system_id")
#     worksheet.write_string(row, col + 1, "loan_id")
#     worksheet.write_string(row, col + 2, "file_name")
#     worksheet.write_string(row, col + 3, "tracking_no")
#     worksheet.write_string(row, col + 4, "allocation_month")
#     worksheet.write_string(row, col + 5, "notice_type")
#     worksheet.write_string(row, col + 6, "notice_id")
#     worksheet.write_string(row, col + 7, "notice_link")
#     worksheet.write_string(row, col + 8, "batch_id")

#     if is_merged_lot and is_linked_loan:
#         worksheet.write_string(row, col + 9, "linked_loan_id")
#         worksheet.write_string(row, col + 10, "merged_lot_file_name")
#     elif is_linked_loan:
#         worksheet.write_string(row, col + 9, "linked_loan_id")
#     elif is_merged_lot:
#         worksheet.write_string(row, col + 9, "merged_lot_file_name")

#     address_headers_set = set()
#     for notice in notices:
#         if notice["primary_address"]:
#             for key in json.loads(notice["primary_address"]):
#                 address_headers_set.add(key)

#     if is_merged_lot and is_linked_loan:
#         start_index = 11
#     elif is_linked_loan or is_merged_lot:
#         start_index = 10
#     else:
#         start_index = 9

#     for i, key in enumerate(address_headers_set):
#         worksheet.write_string(row, col + i + start_index, changeCamelCaseToSnakeCase(key))

#     row += 1

#     for notice in notices:
#         worksheet.write_string(row, col, str(notice["id"]))
#         worksheet.write_string(row, col + 1, "'" + str(notice["loan_id"]))
#         worksheet.write_string(row, col + 2, notice["local_pdf_file_name"])
#         worksheet.write_string(row, col + 3, str(notice["tracking_id"] or ""))
#         worksheet.write_string(row, col + 4, str(notice["allocation_month"] or ""))
#         worksheet.write_string(row, col + 5, str(notice["notice_type"] or ""))
#         worksheet.write_string(row, col + 6, notice["notice_id"])
#         if links_written_in_worksheet < EXCEL_WORKSHEET_HYPERLINK_LIMIT:
#             if str(notice["s3_link"]):
#                 worksheet.write_url(row, col + 7, str(notice["s3_link"] or ""))
#                 links_written_in_worksheet += 1
#         else:
#             worksheet.write_string(row, col + 7, str(notice["s3_link"] or ""))
#         worksheet.write_string(row, col + 8, kwargs["batch_id"])

#         if is_merged_lot and is_linked_loan:
#             worksheet.write_string(row, col + 9, "'" + str(notice["linked_loan_id"] or ""))
#             worksheet.write_string(
#                 row,
#                 col + 10,
#                 str(notice_id_merged_file_name_mapping[int(notice["notice_id"])]) or "",
#             )
#         elif is_linked_loan:
#             worksheet.write_string(row, col + 9, "'" + str(notice["linked_loan_id"] or ""))
#         elif is_merged_lot:
#             worksheet.write_string(
#                 row,
#                 col + 9,
#                 str(notice_id_merged_file_name_mapping[int(notice["notice_id"])]) or "",
#             )

#         if notice["primary_address"]:
#             address = json.loads(notice["primary_address"])
#             for i, key in enumerate(address_headers_set):
#                 worksheet.write_string(row, col + i + start_index, str(address.get(key, "")))
#         row += 1

#     for i, key in enumerate(address_headers_set):
#         try:
#             if "cell_1" in key:
#                 key = key.split("table_1_cell_1_")[1]
#             elif "table_1_" in key:
#                 key = key.split("table_1_")[1]
#             key = key.split("_")[0] if "address" in key else key.replace("_", " ")
#         except Exception as e:
#             logger.error(f"key spilitting error: {str(e)}, {key}")
#         worksheet.write_string(0, 0 + i + start_index, changeCamelCaseToSnakeCase(key))

#     workbook.close()

#     logger.info("notice_callback.commons.generate_tracking_excel :: tracking_excel generated")


# def get_batch_notices(**kwargs):
#     logger.info("notice_callback.commons.get_batch_notices")
#     try:
#         url = f"{NOTICE_SERVICE_BASE_URL.rstrip('/')}/notices"
#         headers = {
#             "authenticationtoken": kwargs.get("token", ""),
#             "role": kwargs.get("role", ""),
#             "Content-type": "application/json",
#             "X-Request-Id": kwargs.get("X-Request-Id", ""),
#             "X-CG-User": json.dumps(kwargs.get("user", {})),
#             "X-CG-Company": json.dumps(kwargs.get("company", {})),
#         }
#         params = {"company_id": kwargs["company_id"], "batch_id": kwargs["batch_id"]}

#         get_resp = requests.get(url=url, params=params, headers=headers)

#         logger.debug(f"notice_callback.commons.get_batch_notices.get_notices.url: {url}")
#         logger.debug(
#             f"notice_callback.commons.get_batch_notices.get_notices.response.status_code: {get_resp.status_code}"
#         )

#     except Exception as e:
#         logger.error(f"notice_callback.commons.get_batch_notices.get_notices.exception: {str(e)}")
#         return str(e)

#     result = json.loads(get_resp.text)
#     return result["data"]


# def send_batch_completion_email(**kwargs):
#     logger.debug("notice_callback.commons.send_batch_completion_email")

#     contactus_link = ""
#     (
#         zip_file_s3_link,
#         tracking_file_s3_link,
#         batch_report_s3_link,
#         s3_tracking_ids_excel_link,
#         notices_zip_generated,
#     ) = (
#         kwargs.get("s3_zip_file_link", ""),
#         kwargs.get("s3_tracking_excel_link", ""),
#         kwargs.get("s3_report_batch_excel_link", ""),
#         kwargs.get("s3_tracking_ids_excel_link", ""),
#         kwargs.get("notices_zip_generated"),
#     )

#     s3_tracking_excel_with_merge_file_name_mapping_link = kwargs.get(
#         "s3_tracking_excel_with_merge_file_name_mapping_link", ""
#     )
#     merging_process_success = kwargs.get("merging_process_success")
#     hyperlink_tracking_file_with_file_name_mapping = ""

#     hyperlink_error_report = ""
#     hyperlink_zip_file = ""
#     hyperlink_tracking_file = ""
#     hyperlink_tracking_ids_file = ""
#     batch_report_data = kwargs["batch_report_data"]
#     failed = batch_report_data["failed"]
#     success = batch_report_data["success"]
#     failed_loans_count = len(failed)
#     succes_loans_count = batch_report_data["success_count"]
#     total_loans_count = failed_loans_count + succes_loans_count

#     notice_type = kwargs.get("notice_type", "Physical")
#     local_zip_file_path = kwargs.get("local_zip_file_path", "")

#     company_details = kwargs["company"]
#     draft_details = kwargs["draft_details"]

#     if kwargs["batch_report_data"]["failed"]:
#         hyperlink_error_report = f"<a href='{batch_report_s3_link}'>Physical Notices failed Report</a>.<br><br>"

#     if kwargs["local_tracking_ids_excel_file_generated_flag"]:
#         hyperlink_tracking_ids_file = (
#             f"<a href='{s3_tracking_ids_excel_link}'>Failed & unused tracking ids</a>.<br><br>"
#         )

#     if merging_process_success:
#         hyperlink_tracking_file_with_file_name_mapping = f"<a href='{s3_tracking_excel_with_merge_file_name_mapping_link}'>Excel file containing notice details with merge notice file mapping</a>.<br><br>"

#     if notices_zip_generated:
#         hyperlink_zip_file = f"<a href='{zip_file_s3_link}'>Zip file containing all notices</a>.<br><br>"
#     if notices_zip_generated:
#         hyperlink_tracking_file = f"<a href='{tracking_file_s3_link}'>Excel file containing notice details</a>.<br><br>"

#     if kwargs["is_linked_loan"]:
#         subject = f"Linked Loan Physical Notice Batch Report for {company_details['company_name']} | notice type - {draft_details['notice_type']} batch id - {kwargs['batch_id']}"
#         starter = f"Linked Loan Physical notices"
#         loans = "linked loans"
#     else:
#         subject = f"Physical Notice Batch Report for {company_details['company_name']} | notice type - {draft_details['notice_type']} batch id - {kwargs['batch_id']}"
#         starter = f"Physical notices"
#         loans = "loans"

#     user_email_payload = {
#         "from_data": {"name": "Credgenics", "email": "legal@credgenics.com"},
#         "subject": f"{subject}",
#         "to_emails": [{"email": kwargs["author"]}],
#         "source": "notice_generation_batch_completion",
#         "email_body": f"""Hi,<br><br>

#         {starter} for {kwargs['batch_id']} have been generated, links for relevant files are provided below.<br><br>

#         company_id: {kwargs.get('company_id')}<br>
#         total {loans}: {total_loans_count}<br>
#         succes {loans}: {succes_loans_count}<br>
#         failed {loans}: {failed_loans_count}<br><br>

#         {hyperlink_zip_file}
#         {hyperlink_tracking_file}
#         {hyperlink_tracking_file_with_file_name_mapping}
#         {hyperlink_error_report}
#         {hyperlink_tracking_ids_file}


#         <a href="{contactus_link}">Contact Us</a><br><br>

#         This email was sent to you by Credgenics (Copyright © 2020 Analog Legalhub Technology Solutions Pvt. Ltd. All Rights Reserved).<br>
#         This is a computer generated email. Please do not reply to this email. """,
#     }

#     send_email(
#         user_email_payload,
#         kwargs["token"],
#         kwargs["role"],
#         kwargs["batch_id"],
#         kwargs["user"],
#         company_details,
#     )

#     credgengics_physical_notice_email_payload = {
#         "from_data": {"name": "Credgenics", "email": "legal@credgenics.com"},
#         "subject": f"{subject}",
#         "to_emails": [{"email": "physical.notice@credgenics.com"}],
#         "source": "notice_generation_batch_completion",
#         "email_body": f"""Hi,<br><br>

#         {starter} for {kwargs['batch_id']} have been generated, links for relevant files are provided below.<br><br>

#         company_id: {kwargs.get('company_id')}<br>
#         total {loans}: {total_loans_count}<br>
#         succes {loans}: {succes_loans_count}<br>
#         failed {loans}: {failed_loans_count}<br><br>

#         {hyperlink_zip_file}
#         {hyperlink_tracking_file}
#         {hyperlink_tracking_file_with_file_name_mapping}
#         {hyperlink_error_report}
#         {hyperlink_tracking_ids_file}

#         <a href="{contactus_link}">Contact Us</a><br><br>

#         This email was sent to you by Credgenics (Copyright © 2020 Analog Legalhub Technology Solutions Pvt. Ltd. All Rights Reserved).<br>
#         This is a computer generated email. Please do not reply to this email. """,
#     }
#     send_email(
#         credgengics_physical_notice_email_payload,
#         kwargs["token"],
#         kwargs["role"],
#         kwargs["batch_id"],
#         kwargs["user"],
#         company_details,
#     )


# def send_email(payload, token, role, request_id, user, company):
#     logger.info("notice_callback.commons.send_email")
#     base_url = COMMUNICATION_SERVICE_BASE_URL
#     url = f"{base_url}/mail"
#     logger.debug(f"notice_callback.commons.send_email.url : {url}")
#     payload["module"] = "notice"

#     headers = {
#         "authenticationtoken": token,
#         "role": role,
#         "accept": "application/json",
#         "Content-Type": "application/json",
#         "X-Request-Id": str(request_id),
#         "X-CG-User": json.dumps(user),
#         "X-CG-Company": json.dumps(company),
#     }

#     result = requests.request(
#         method="POST",
#         url=url,
#         headers=headers,
#         json=payload,
#     )
#     logger.debug(f"notice_callback.commons.send_email.response.status_code : {result.status_code}")

#     if result.status_code != 200:
#         logger.debug(f"notice_callback.commons.send_email.response.err : {result.text}")
#         return json.loads(result.text)

#     data = json.loads(result.text)

#     if data:
#         email_data = data.get("data")
#         unique_mail_id = email_data.get("unique_mail_id")
#         status = data.get("output")

#         return (status, unique_mail_id)
#     return None, None


# def generate_unused_tracking_ids_excel(**kwargs):
#     logger.info("notice_callback.commons.generate_unused_tracking_ids_excel")

#     # EXTRACT DATA
#     local_batch_report_file_path = kwargs["local_tracking_ids_excel_path"]
#     failed_tracking_ids_key = f'failed_tracking_id_{kwargs.get("batch_id","")}'
#     failed_tracking_ids = pop_all_values_from_redis_key(failed_tracking_ids_key)
#     unused_tracking_ids_key = f"physical_notice_tracking_ids_{kwargs['batch_id']}"
#     unused_tracking_ids = pop_all_values_from_redis_key(unused_tracking_ids_key)

#     # GENERATE EXCEL
#     if failed_tracking_ids or unused_tracking_ids:
#         workbook = Workbook(filename=local_batch_report_file_path)
#         worksheet = workbook.add_worksheet()
#         # headings
#         row = 0
#         column = 0
#         bold = workbook.add_format({"bold": True})
#         worksheet.write(row, column, "tracking_id", bold)
#         worksheet.write(row, column + 1, "failed/unused", bold)
#         worksheet.set_column(0, 1, len("failed/unused"))
#         row = row + 1

#         if failed_tracking_ids:
#             for id in failed_tracking_ids:
#                 worksheet.write(row, column, str(id))
#                 worksheet.write(row, column + 1, "failed")
#                 row = row + 1

#         if unused_tracking_ids:
#             for id in unused_tracking_ids:
#                 worksheet.write(row, column, str(id))
#                 worksheet.write(row, column + 1, "unused")
#                 row = row + 1

#         workbook.close()
#         return True
#     return False


# def generate_failed_notices_report_excel(**kwargs):
#     logger.info("notice_callback.commons.generate_failed_notices_report_excel")

#     local_batch_report_file_path = kwargs["local_batch_report_file_path"]
#     batch_report_data = get_batch_report_data(**kwargs)
#     workbook = Workbook(filename=local_batch_report_file_path)
#     worksheet = workbook.add_worksheet()
#     failed = batch_report_data["failed"]

#     if failed:
#         row = 0
#         col = 0
#         cell0_0 = "linked_loan_id" if "linked_loan" in kwargs["type_of_task"] else "loan_id"
#         headers = ["loan_id", "reason", "status_code", "notice_id"]
#         for i, key in enumerate(headers):
#             worksheet.write_string(row, col + i, key)
#         row += 1
#         failed_records = []
#         for record in failed:
#             if len(record.get("notice_id_list", [])):
#                 for notice_id in record["notice_id_list"]:
#                     failed_record = {
#                         "reason": record["reason"],
#                         "loan_id": record["loan_id"],
#                         "status_code": record["status_code"],
#                         "notice_id": notice_id,
#                     }
#                     failed_records.append(failed_record)
#             else:
#                 failed_records.append({**record, "notice_id": None})

#         for record in failed_records:
#             for i, key in enumerate(headers):
#                 if key == "loan_id":
#                     worksheet.write_string(row, col + i, "'" + str(record.get(key, "")))
#                 else:
#                     worksheet.write_string(row, col + i, str(record.get(key, "")))
#             row += 1
#         worksheet.write_string(0, 0, cell0_0)
#         workbook.close()
#     return batch_report_data


# def get_batch_report_data(**kwargs):
#     logger.info("notice_callback.commons.get_batch_report_data")

#     batch_id = kwargs["batch_id"]
#     batch_status = {}
#     success = []
#     failed = []

#     execution_data_url = f"{QUEUE_SERVICE_BASE_URL}/batch_data"
#     req_body = {
#         "type": "notice",
#         "batch_id": batch_id,
#         "channel": "physical" if kwargs["is_linked_loan"] else "linked_loan_physical",
#     }

#     logger.debug(f"notice_callback.commons.get_batch_report_data.url : {execution_data_url}")

#     try:
#         execution_data = requests.request(
#             method="POST",
#             url=execution_data_url,
#             headers={
#                 "X-CG-User": json.dumps(kwargs.get("user", {})),
#                 "X-Request-Id": kwargs.get("X-Request-Id", ""),
#                 "X-CG-Company": json.dumps(kwargs.get("company", {})),
#             },
#             json=req_body,
#         )
#         logger.debug(f"notice_callback.commons.get_batch_report_data.status_code : {execution_data.status_code}")

#     except Exception as e:
#         logger.error(f"execution_data.exception_{batch_id} :: {str(e)}")
#         raise Exception(str(e))

#     if execution_data.status_code != 200:
#         logger.error(f"execution_data.text_{batch_id} :: {execution_data.text}")
#         raise Exception(f"batch_data api failed with status_code {execution_data.status_code}")
#         # self.retry(queue=queue, kwargs=kwargs, countdown=20, max_retries=5)

#     execution_data = execution_data.json()
#     if not execution_data["output"]["batch_data"]:
#         logger.error(f"get_batch_report_data.batch_data_api.exception: batch_data is empty")
#         raise Exception(f"batch_data api gave empty response with status_code {execution_data.status_code}")

#     batch_loans_data = execution_data["output"]["batch_loans_data"]
#     failed_loans_data = []
#     success_loans_data = []

#     for data in batch_loans_data:
#         if data["status"] == "SUCCESS":
#             success_loans_data.append(data)
#         else:
#             failed_loans_data.append(data)

#     # If none of the loan_id failed
#     if len(batch_loans_data) == 0:
#         success = json.loads(execution_data["output"]["batch_data"][0]["loan_data"])
#     else:
#         for failed_data in failed_loans_data:
#             response = json.loads(failed_data.get("response"))
#             message = response.get("message", "")
#             notice_id_list = response.get("notice_id_list", [])
#             status_code = failed_data.get("status_code")
#             loan_id = failed_data.get("loan_id")
#             failed.append(
#                 {
#                     "reason": str(message),
#                     "loan_id": loan_id,
#                     "status_code": status_code,
#                     "notice_id_list": notice_id_list,
#                 }
#             )
#     total_loans = int(execution_data["output"]["batch_data"][0]["total_loans"])

#     batch_status["success"] = success
#     batch_status["failed"] = failed
#     batch_status["success_count"] = total_loans - len(failed)

#     del failed_loans_data
#     del success_loans_data
#     del batch_loans_data
#     return batch_status


# def delete_redis_keys(batch_id):
#     redis_instance = redis.Redis(host=REDIS["HOST"], port=REDIS["PORT"])
#     keys_list = [
#         f"notice_kwargs_{batch_id}",
#         f"notice_draft_{batch_id}",
#         f"callback_kwargs_{batch_id}",
#         f"initial_notice_id_{batch_id}",
#     ]
#     redis_instance.delete(*keys_list)
#     redis_instance.close()
#     return


# def pop_all_values_from_redis_key(key_name):
#     redis_instance = redis.Redis(host=REDIS["HOST"], port=REDIS["PORT"])
#     val_list = redis_instance.lrange(key_name, 0, -1)
#     if len(val_list) > 0:
#         val_list = [item.decode("utf-8") for item in val_list]
#         return val_list
#     redis_instance.close()
#     return None


# def get_notice_id_and_loan_id_mappings(notice_data: list, is_linked_loan: bool, initial_notice_id: int, batch_id: str):
#     """
#     Method to get these mappings -
#         {loan_id : [notice_id]}
#         {notice_id : loan_id}
#         sorted [loan_id] acc to notice_id ASC
#     """
#     logger.info("notice_callback.commons.get_notice_id_loan_id_mapping")
#     # {loan_id : [notice_id]}
#     loan_id_notice_id_list_map = {}
#     # {notice_id : loan_id}
#     notice_id_loan_id_mapping = {}

#     redis_instance = redis.Redis(host=redis_conf["HOST"], port=redis_conf["PORT"])
#     last_notice_id = int(redis_instance.get(f"initial_notice_id_redis_key_{batch_id}"))
#     redis_instance.close()

#     if is_linked_loan:
#         loan_id_key = "linked_loan_id"
#     else:
#         loan_id_key = "loan_id"

#     for el in notice_data:
#         notice_id_loan_id_mapping[int(el["notice_id"])] = el[loan_id_key]

#         if loan_id_notice_id_list_map.get(el[loan_id_key]) is None:
#             loan_id_notice_id_list_map[el[loan_id_key]] = {int(el["notice_id"])}
#         else:
#             loan_id_notice_id_list_map[el[loan_id_key]].add(int(el["notice_id"]))

#     # sorted [loan_id] acc to notice_id ASC
#     loan_id_list_sorted_acc_to_notice_id = []
#     i = -1

#     for notice_id in range(initial_notice_id, last_notice_id):
#         loan_id = notice_id_loan_id_mapping.get(notice_id)

#         if loan_id:
#             if i >= 0:
#                 if loan_id != loan_id_list_sorted_acc_to_notice_id[i]:
#                     loan_id_list_sorted_acc_to_notice_id.append(loan_id)
#                     i = i + 1
#             else:
#                 loan_id_list_sorted_acc_to_notice_id.append(loan_id)
#                 i = i + 1

#     logger.info("notice_callback.commons.get_notice_id_loan_id_mapping completed")
#     return (
#         loan_id_notice_id_list_map,
#         notice_id_loan_id_mapping,
#         loan_id_list_sorted_acc_to_notice_id,
#     )


# def merge_pdf_of_individual_lot(
#     notice_ids_to_merged,
#     local_pdf_directory_path,
#     local_merged_pdf_directory_path,
#     merged_file_name,
# ):
#     """
#     # MERGED File Name:
#     # <notice_type>_<merged serial number>_<first notice id>_<last notice id>
#     """
#     logger.info("notice_callback.commons.merge_pdf_of_individual_lot")
#     file_list = notice_ids_to_merged

#     logger.debug(f"Merging files, processing lot : {notice_ids_to_merged}")

#     with fitz.open() as merge_file_result:
#         for file in file_list:
#             file_name = local_pdf_directory_path + "/" + str(file) + ".pdf"
#             try:
#                 with fitz.open(file_name) as mfile:
#                     merge_file_result.insert_pdf(mfile)
#             except Exception as e:
#                 logger.error(f"Exception occured while merging the pdf files : {e}")
#                 return False
#         merge_file_result.save(local_merged_pdf_directory_path + "/" + merged_file_name)
#         logger.debug(f"Merging files success, processed lot : {notice_ids_to_merged}")

#     return True


# def create_merge_list_batches(
#     loan_id_notice_id_list_map,
#     loan_id_list_sorted_acc_to_notice_id,
#     custom_batch_size,
# ):
#     """
#     CREATE BATCHES FOR MERGING NOTICEs
#     """
#     logger.info("notice_callback.commons.create_merge_list_batches")
#     merge_file_list = [[]]
#     idx = 0

#     for loan_id in loan_id_list_sorted_acc_to_notice_id:
#         notice_id_list = list(loan_id_notice_id_list_map[loan_id])
#         notice_id_list.sort()
#         if len(notice_id_list) > custom_batch_size:
#             merge_file_list.append(notice_id_list)
#             idx = idx + 1
#         elif len(merge_file_list[idx]) + len(notice_id_list) <= custom_batch_size:
#             merge_file_list[idx].extend(notice_id_list)
#         else:
#             merge_file_list.append([])
#             idx = idx + 1
#             merge_file_list[idx].extend(notice_id_list)

#     logger.debug(f"tasks.notice.notice_callbacks_commons.create_merge_list_batches: {merge_file_list}")
#     return merge_file_list


# def get_merged_file_name_mapping(notice_type, merged_file_list):
#     """
#     METHOD to get merged file name mappings for each notice_id to be put in tracking_id excel
#     """
#     logger.info("notice_callback.commons.get_merged_file_name_mapping")
#     map_file_name = {}
#     for count, arr in enumerate(merged_file_list):
#         file_name = notice_type + "_" + str(count + 1) + "_" + str(arr[0]) + "_" + str(arr[len(arr) - 1]) + ".pdf"
#         for notice_id in arr:
#             map_file_name[notice_id] = file_name

#     logger.debug(f"tasks.notice.notice_callbacks_commons.get_merged_file_name_mapping: {map_file_name}")
#     return map_file_name


# async def create_merge_pdf_and_zip(**kwargs):
#     """
#     METHOD TO MERGE NOTICEs AND ZIP THEM
#     """
#     logger.info("tasks.notice.notice_callback.commons.create_merge_pdf_and_zip")
#     logger.debug(f"tasks.notice.notice_callback.commons.create_merge_pdf_and_zip kwargs : {kwargs}")

#     local_merged_pdf_directory_path = kwargs["local_merged_pdf_directory_path"]
#     local_pdf_directory_path = kwargs["local_pdf_directory_path"]

#     local_pdf_directory_path = kwargs["local_pdf_directory_path"]
#     local_merged_pdf_directory_path = kwargs["local_merged_pdf_directory_path"]
#     local_merged_pdf_zip_file_path = kwargs["local_merged_pdf_zip_file_path"]

#     merged_lot_size = kwargs.get("merged_lot_size")
#     notice_data = kwargs["notice_data"]
#     notice_type = kwargs["notice_type"]
#     is_linked_loan = kwargs["is_linked_loan"]
#     initial_notice_id = kwargs["initial_notice_id"]
#     batch_id = kwargs["batch_id"]
#     count_of_notices = len(notice_data)
#     batch_name = kwargs.get("batch_name")
#     request_id = kwargs["request_id"]
#     local_batch_directory_path = kwargs["local_batch_directory_path"]

#     if batch_name:
#         batch_name = format_batch_name(batch_name)
#         merged_lot_zip_name = f"{batch_name}_{batch_id}_{count_of_notices}"
#     else:
#         merged_lot_zip_name = f"{batch_id}_{count_of_notices}"

#     # GET {notice_id: loan_id}, {loan_id:[notice_id]} & [loan_id]
#     (
#         loan_id_notice_id_list_map,
#         notice_id_loan_id_map,
#         loan_id_list_sorted_acc_to_notice_id,
#     ) = get_notice_id_and_loan_id_mappings(notice_data, is_linked_loan, initial_notice_id, batch_id)
#     logger.info("tasks.notice.notice_callback.commons.merge_pdf notice_id:loan_id mappings created")

#     # CREATE BATCHES FOR MERGING NOTICES :
#     merge_file_list_batches = create_merge_list_batches(
#         loan_id_notice_id_list_map,
#         loan_id_list_sorted_acc_to_notice_id,
#         merged_lot_size,
#     )
#     logger.info("tasks.notice.notice_callback.commons.create_merge_pdf_and_zip merge lists created")

#     # CREATE FILE NAME MAPPING WITH LOAN_IDS :
#     notice_id_merged_file_name_mapping = get_merged_file_name_mapping(notice_type, merge_file_list_batches)
#     logger.info("tasks.notice.notice_callback.commons.create_merge_pdf_and_zip file name mappings created")

#     logger.info("tasks.notice.notice_callback.commons.create_merge_pdf_and_zip merging pdfs now")

#     try:
#         kafka_producer = JSONProducer(client_id=MERGE_NOTICE_CLIENT_ID)
#         logger.debug(
#             "app.tasks.notice.notice_callback.commons.create_merge_pdf_and_zip || Notice_merge producer initialised"
#         )
#         await kafka_producer.start()
#         logger.debug(
#             "app.tasks.notice.notice_callback.commons.create_merge_pdf_and_zip || Notice_merge producer started"
#         )
#     except Exception as e:
#         logger.error(
#             f"app.tasks.notice.notice_callback.commons.create_merge_pdf_and_zip || kafkaProducer.err :: {str(e)}"
#         )
#         logger.error(
#             "app.tasks.notice.notice_callback.commons.create_merge_pdf_and_zip || Messages won't be produced to merge_pdf topic"
#         )
#         return notice_id_merged_file_name_mapping

#     # CREATE EVENTS FOR MERGE NOTICE HERE ---
#     try:
#         remaining_tasks_count_key = f"{batch_id}_remaining_tasks"
#         merge_pdf_success_redis_key = f"{batch_id}_merge_pdf_success"

#         redis_ = redis.Redis(host=REDIS["HOST"], port=REDIS["PORT"])
#         # adding "~" because of namespace at cg_redis on notice_merge consumer end
#         redis_.set("~" + remaining_tasks_count_key, len(merge_file_list_batches))
#         redis_.set("~" + merge_pdf_success_redis_key, 1)

#         logger.debug(
#             "app.tasks.notice.notice_callback.commons.create_merge_pdf_and_zip || Writing messages for notice_merge topic"
#         )
#         merged_pdf_count = len(merge_file_list_batches)
#         for idx, notice_ids_to_merged in enumerate(merge_file_list_batches):
#             merged_file_name = notice_id_merged_file_name_mapping[notice_ids_to_merged[0]].strip()
#             message = {
#                 "notice_ids_to_merged": notice_ids_to_merged,
#                 "local_pdf_directory_path": local_pdf_directory_path,
#                 "local_merged_pdf_directory_path": local_merged_pdf_directory_path,
#                 "local_zip_file_path": local_merged_pdf_zip_file_path,
#                 "merged_file_name": merged_file_name,
#                 "total_files": merged_pdf_count,
#                 "local_batch_directory_path": local_batch_directory_path,
#                 "batch_id": batch_id,
#                 "notice_type": notice_type,
#                 "zip_file_name": "physical_notice_zip/" + merged_lot_zip_name,
#                 "batch_name": batch_name,
#                 "remaining_tasks_count": remaining_tasks_count_key,
#                 "merge_pdf_success_redis_key": merge_pdf_success_redis_key,
#                 "request_id": request_id,
#                 "user": kwargs.get("user"),
#                 "company": kwargs.get("company"),
#                 "merge_called_through_kafka": False,
#             }
#             await kafka_producer.send(
#                 topic=MERGE_NOTICE_TOPIC,
#                 value=message,
#                 partition=idx % int(NUMBER_OF_PARTITIONS_FOR_MERGE_NOTICE_TOPIC),
#             )

#         await kafka_producer.stop()
#     except Exception as e:
#         logger.error(
#             f"app.tasks.notice.notice_callback.commons.create_merge_pdf_and_zip || Error while producing message or writing to topic :: {str(e)}"
#         )

#     return notice_id_merged_file_name_mapping


# def format_batch_name(batch_name):
#     logger.info("tasks.notice.notice_callback.commons.format_batch_name")
#     special_characters = ["!", "-", "_", ".", "*", "'", "(", ")"]
#     new_batch_name = ""
#     replace_character = MERGE_FILE_NAME_REPLACE_CHARACTER
#     for i in batch_name:
#         if (
#             not ((i >= "a" and i <= "z") or (i >= "A" and i <= "Z") or (i >= "0" and i <= "9"))
#             and i not in special_characters
#         ):
#             new_batch_name += replace_character
#         else:
#             new_batch_name += i
#     return new_batch_name



# import json
# import logging
# import redis
# import requests


# from ...utils import get_s3_client
# from ...settings import (
#     REDIS,
#     QUEUE_SERVICE_BASE_URL,
# )

# logger = logging.getLogger(__name__)
# logger.setLevel(logging.DEBUG)

# redis_conf = REDIS
# redis_obj = redis.Redis(host=redis_conf["HOST"], port=redis_conf["PORT"])

# s3 = get_s3_client()


# def logs_tracker(response, request_id):
#     url = f"{QUEUE_SERVICE_BASE_URL}/save_batch_response"
#     user = response.get("kwargs").get("user")
#     company = response.get("kwargs").get("company")
#     response_kwargs = response["kwargs"]
#     batch_id = response_kwargs["batch_id"]
#     req_body = {
#         "response": {
#             "response": json.dumps(response["response"]),
#             "batch_id": str(batch_id),
#             "status": str(response["status"]),
#             "status_code": str(response["status_code"]),
#             "loan_id": str(response_kwargs["loan_id"]),
#             "company_id": company["company_id"],
#         },
#         "payload": {"batch_id": batch_id, "type": "notice"},
#     }
#     try:
#         res = requests.request(
#             method="POST",
#             url=url,
#             json=req_body,
#             headers={
#                 "Content-Type": "application/json",
#                 "X-Request-Id": request_id,
#                 "X-CG-User": json.dumps(user),
#                 "X-CG-Company": json.dumps(company),
#             },
#         )
#         logger.info(f"physical_notice_generation.execution_response_update.status_code :: {res.status_code}")
#         logger.debug(f"helper.logs_tracker.res : {res.text}")
#     except Exception as e:
#         logger.info(f"physical_notice_generation.execution_response_update.exception :: {str(e)}")
#         return False
#     if res.status_code not in (200, 201):
#         logger.info(f"physical_notice_generation.execution_response_update.status :: str({res.text})")
#         return False
#     return True


# def map_formatter(loan_address_map, address_file_map, files_s3_map):
#     logger.info("notice.map_formatter")

#     final_map_list = []
#     for loan_id, address_list in loan_address_map.items():
#         # https://credgenics.atlassian.net/browse/POD2-1691
#         # creating address_set here to make sure that the notices on the same combination of ("Name1 X Property Address1") are not duplicated in the report
#         address_set = set()
#         for addresses in address_list:
#             address = addresses[0]
#             if address in address_set:
#                 continue
#             else:
#                 address_set.add(address)
#             if address_file_map.get(address):
#                 final_map = {}
#                 final_map["loan_id"] = loan_id
#                 filename = address_file_map[address].get("filename")
#                 final_map["local_pdf_file_name"] = filename
#                 final_map["primary_address"] = address_file_map[address].get("primary_address")
#                 final_map["tracking_id"] = address_file_map[address].get("tracking_id")
#                 final_map["s3_link_uuid"] = files_s3_map.get(filename, ["", ""])[0]
#                 final_map["s3_link"] = files_s3_map.get(filename, ["", ""])[1]
#                 final_map["created"] = str(address_file_map[address].get("created"))
#                 final_map["borrower_type"] = addresses[1]
#                 final_map["notice_id"] = str(address_file_map[address].get("notice_id"))
#                 final_map["address_type"] = addresses[2]
#                 final_map_list.append(final_map)
#     return final_map_list


# def get_draft_fields(**kwargs):
#     logger.info("tasks.notice.helper.get_draft_fields")
#     local_batch_directory_path = kwargs["local_batch_directory_path"]
#     local_draft_file_path = kwargs["local_draft_file_path"]
#     BUCKET_NAME = kwargs["BUCKET_NAME"]
#     company_id = kwargs["company_id"]
#     draft_id = kwargs["draft_id"]
#     os.makedirs(local_batch_directory_path, exist_ok=True)

#     try:
#         s3.download_file(
#             BUCKET_NAME,
#             "drafts/" + str(company_id) + "/" + draft_id + ".docx",
#             local_draft_file_path,
#         )
#     except Exception as e:
#         logger.error(f"tasks.notice.helpers.get_draft_fields.download_exception: {str(e)}")
#         return "Unable to download draft from s3"

#     if os.path.exists(local_draft_file_path):
#         try:
#             document = MailMerge(local_draft_file_path)
#             draft_fields = list(document.get_merge_fields())
#             document.close()
#             return {draft_id: draft_fields}
#         except Exception as e:
#             logger.error(f"get_draft_fields.mailmerge_exception: {str(e)}")
#             return f"get_draft_fields.mailmerge_exception: {str(e)}"
#     else:
#         logger.debug(f"get_draft_fields.local_path_err: Draft not present at location {local_draft_file_path}")
#         return f"Draft not present at location {local_draft_file_path}"


# def make_notice_doc_file(**kwargs):
#     logger.info("notice.helpers.make_notice_doc_file")
#     logger.info(f"tasks.notice.make_notice_doc_file.batch_id :: {kwargs['batch_id']}")

#     data = kwargs["notice_data"]
#     changed_dt = data["changed_dt"]
#     loan_id = changed_dt["loan_ids"]
#     table_dict = data.get("master_table_dict", "")
#     index = kwargs.get("index", "")
#     local_draft_file_path = kwargs["local_draft_file_path"]
#     local_doc_directory_path = kwargs["local_doc_directory_path"]
#     local_pdf_directory_path = kwargs["local_pdf_directory_path"]
#     local_qr_code_directory_path = kwargs["local_qr_code_directory_path"]
#     local_merged_pdf_directory_path = kwargs["local_merged_pdf_directory_path"]
#     notice_id = kwargs.get("notice_id", "")
#     type_of_task = kwargs["type_of_task"]

#     try:
#         if not os.path.exists(local_draft_file_path):
#             logger.error(f"notice.make_notice_doc_file.local_draft_file_path.err : local draft file doesn't exist")
#             raise Exception(f"Could not locate draft on the local file system")

#         # Axis bank condition
#         if kwargs["company_id"] == "d2d50f3e-4b79-40d7-b9dd-6cadb0b30141" and changed_dt["loan_type"] in [
#             "Credit Card"
#         ]:
#             changed_dt["loan_id"] = changed_dt["credit_card_number"]

#         document = MailMerge(local_draft_file_path)

#         if table_dict:
#             for key, val in table_dict.items():
#                 document.merge_rows(key, val)

#         if type_of_task == "physical":
#             changed_dt["notice_id"] = str(notice_id)

#         document.merge(**changed_dt)

#         os.makedirs(local_doc_directory_path, exist_ok=True)
#         os.makedirs(local_pdf_directory_path, exist_ok=True)
#         os.makedirs(local_qr_code_directory_path, exist_ok=True)

#         if local_merged_pdf_directory_path:
#             os.makedirs(local_merged_pdf_directory_path, exist_ok=True)

#         if type_of_task == "digital":
#             if kwargs.get("linked_loan_id"):
#                 loan_id = kwargs["linked_loan_id"]
#             loan_id = loan_id.replace("/", "_") if "/" in loan_id else loan_id
#             filename = loan_id.strip().replace(" ", "_") + "_" + str(index)
#         elif type_of_task == "physical":
#             filename = str(notice_id).strip().replace(" ", "_")

#         input_file = os.path.join(local_doc_directory_path, f"{filename}.docx")
#         output_file = os.path.join(local_pdf_directory_path, f"{filename}.pdf")

#         document.write(input_file, kwargs.get("is_vernacular", ""))
#         document.close()

#         notice_file_data = {
#             "input_file_path": input_file,
#             "output_file_path": output_file,
#             "filename": filename,
#         }
#         logger.debug(f"helpers.make_notice_doc_file.notice_file_data : {notice_file_data}")
#         return notice_file_data
#     except Exception as e:
#         raise Exception(f"make_notice_doc_file.exception: {str(e)}")


# def doc2pdf_converter(output_folder, **kwargs):
#     logger.info("notice.doc2pdf_converter")
#     logger.info(f"tasks.notice.doc2pdf_converter.batch_id :: {kwargs['batch_id']}")
#     logger.debug(
#         f"input_file: '{kwargs['input_file']}', output_file: '{kwargs['output_file']}', author: '{kwargs['author']}'"
#     )

#     author = kwargs["author"]
#     input_file, output_file = kwargs["input_file"], kwargs["output_file"]

#     random_instance_id = uuid.uuid4().hex
#     try:
#         os.makedirs(output_folder, exist_ok=True)
#         folder = output_folder
#         # random_instance_number = random.randint(0,10000000000000000)

#         # cmd_output = run(
#         #     [
#         #         "/Applications/LibreOffice.app/Contents/MacOS/soffice",
#         #         f"-env:UserInstallation=file:///tmp/LibreOffice_Conversion_{random_instance_id}{author}",
#         #         "--headless",
#         #         "--convert-to",
#         #         "pdf:writer_pdf_Export",
#         #         "--outdir",
#         #         f"{folder}",
#         #         f"{input_file}",
#         #     ],
#         #     capture_output=True,
#         # )

#         cmd_output = run(
#             [
#                 "soffice",
#                 f"-env:UserInstallation=file:///tmp/LibreOffice_Conversion_{random_instance_id}{author}",
#                 "--headless",
#                 "--convert-to",
#                 "pdf:writer_pdf_Export",
#                 "--outdir",
#                 f"{folder}",
#                 f"{input_file}",
#             ],
#             capture_output=True,
#         )

#         stdout, stderr = cmd_output.stdout, cmd_output.stderr
#         try:
#             shutil.rmtree(f"/tmp/LibreOffice_Conversion_{random_instance_id}{author}")
#         except Exception as e:
#             logger.error(f"soffice env removal error - {e}")

#         if stderr:
#             logger.error(f"doc2pdf_converter.stderr: {stderr}")
#             logger.debug(f"doc2pdf_converter.stdout: {stdout}")
#             if "source file" in str(stderr):
#                 raise Exception(f"stderr - {stderr}")

#         if not stdout:
#             raise Exception(f"no stdout err- {stderr}")

#         if not os.path.exists(output_file):
#             logger.error("doc2pdf_converter.err : Abort Code:27 error")
#             raise Exception("Abort Code:27 error, output_file doesn't exist")
#     except Exception as e:
#         logger.error(f"doc2pdf_converter.exception: {str(e)}, random_instance_id :: {random_instance_id}")
#         raise Exception(f"doc2pdf_converter.exception: {str(e)}")


# def get_dsc_on_notice_pdf(**kwargs):
#     logger.info("notice.get_dsc_on_notice_pdf")
#     logger.info(f"tasks.notice.get_dsc_on_notice_pdf.batch_id :: {kwargs['batch_id']}")

#     pdf_file_path = kwargs["output_file"]
#     dsc_placement = kwargs["dsc_placement"]
#     filename_with_extension = pdf_file_path.split("/")[-1]
#     filename = filename_with_extension.split(".pdf")[0]

#     try:
#         with open(pdf_file_path, "rb") as pdf_file:
#             encoded_string = base64.b64encode(pdf_file.read())

#         timestamp = datetime.now().strftime("%d%m%Y%H:%M:%S")
#         cs = hashlib.md5(f"{TRUECOPY_API_KEY}{timestamp}".encode()).hexdigest()[0:16]

#         url = TRUECOPY_BASE_URL

#         headers = {
#             "Content-Type": "application/json",
#         }

#         if dsc_placement == "all":
#             dsc_placement_value = "[410:110]"
#         elif dsc_placement == "signature":
#             dsc_placement_value = "[410:110]"
#         else:
#             return False

#         req_payload = {
#             "pfxid": TRUECOPY_ID,
#             "pfxpwd": TRUECOPY_PWD,
#             "accessid": TRUECOPY_ACCESS_ID,
#             "timestamp": timestamp,
#             "cs": cs,
#             "filename": filename + "_signed.pdf",
#             "signloc": dsc_placement_value,
#             "filepwd": "",
#             "contents": encoded_string,
#         }

#         response = requests.post(url=url, json=req_payload, headers=headers, timeout=300)
#         encoded_response = response.text
#         logger.debug(f"True copy api response status code: {response.status_code}")
#         if response.status_code != 200:
#             logger.debug(f"True copy api response: {response.text}")
#         b64_decoded_response = base64.b64decode(encoded_response, validate=True)
#         pdf_file = open(pdf_file_path, "wb")
#         pdf_file.write(b64_decoded_response)
#         pdf_file.close()
#     except Exception as e:
#         logger.error(f"notice.helpers.get_dsc_on_notice_pdf.exception :: {str(e)}")
#         raise Exception(f"notice.helpers.get_dsc_on_notice_pdf.exception :: {str(e)}")


# def generate_notice_for_all_addresses(**kwargs):
#     logger.info("notice.generate_notice_for_all_addresses")
#     logger.info(f"notice.generate_notice_for_all_addresses.batch_id :: {kwargs['batch_id']}")

#     preview = kwargs.get("preview", False)
#     notice_data = kwargs.get("notice_data", "")
#     master_table_dict = notice_data["master_table_dict"]
#     duplicated_tables_dict = notice_data["duplicated_tables_dict"]
#     local_pdf_directory_path = kwargs["local_pdf_directory_path"]
#     address_file_map = kwargs["address_file_map"]
#     draft_fields = kwargs.get("draft_fields", [])
#     table_1_fields = []

#     for i in range(len(draft_fields)):
#         if "table_1" in draft_fields[i]:
#             table_1_fields.append(draft_fields[i])

#     index_p = 0
#     tracking_ids_batch = []
#     failed_tracking_id_list = []

#     # total_length = 0

#     # for key, val in master_table_dict.items():
#     #     list_len = len(val)
#     #     total_length += list_len

#     address_tables = kwargs["notice_data"]["address_tables"]
#     if "property" in address_tables:
#         address_tables.remove("property")
#     if "auctioned" in address_tables:
#         address_tables.remove("auctioned")
#     borrower_address_count = 0
#     for key, value in master_table_dict.items():
#         borrower_type = key.split("_")[4]
#         if borrower_type == "co":
#             borrower_type += "_applicant"
#         if borrower_type in address_tables:
#             borrower_address_count += len(value)

#     redis_instance = redis.Redis(host=REDIS["HOST"], port=REDIS["PORT"])
#     next_initial_notice_id = redis_instance.incrby(kwargs["initial_notice_id_redis_key"], borrower_address_count)
#     redis_instance.close()

#     initial_notice_id = next_initial_notice_id - 1
#     notice_id_list = [
#         notice_id for notice_id in range(initial_notice_id - borrower_address_count + 1, next_initial_notice_id)
#     ]
#     total_length = len(notice_id_list)
#     failed_tracking_ids_key = f'failed_tracking_id_{kwargs.get("batch_id","")}'

#     if kwargs["use_tracking_ids"]:
#         for i in range(total_length):
#             physical_notice_trackings_ids_redis_key = kwargs.get("physical_notice_trackings_ids_redis_key", "")
#             tracking_id = get_next_tracking_id(physical_notice_trackings_ids_redis_key)
#             if tracking_id:
#                 tracking_ids_batch.append(tracking_id)
#             else:
#                 break

#         if total_length != len(tracking_ids_batch):
#             logger.error(f"notice.generate_notice_for_all_addresses.tracking_ids.err : Not enough tracking ids")
#             return f"Insufficient tracking ids. Atleast required: {total_length}", notice_id_list, tracking_ids_batch

#     try:
#         for key, val in master_table_dict.items():
#             table_reference = key.split("_", 2)
#             table_reference = table_reference[0] + "_" + table_reference[1]
#             if table_reference == "table_1":
#                 primary_cell = key
#                 break

#         file_name_sequence_counter = total_length

#         notice_id = initial_notice_id
#         for key in reversed(master_table_dict):
#             val = master_table_dict[key]

#             if val:
#                 val.reverse()
#             table_reference = key.split("_", 2)
#             table_reference = table_reference[0] + "_" + table_reference[1]

#             borrower_type = key.split("_", 4)[-1].split("_", 1)[0]
#             if borrower_type == "co":
#                 borrower_type = "co_applicant"
#             borrower_type += "_"

#             primary_address_dt = {}
#             temp_address = {}
#             length = len(val) if val else 0
#             for index in range(length):
#                 tracking_id = ""
#                 for k, v in val[index].items():
#                     if table_reference in k and v not in [None, ""]:
#                         temp = k.split(borrower_type)
#                         try:
#                             temp = temp[0] + temp[1]
#                         except:
#                             temp = temp[0]
#                         temp = temp.replace(table_reference, "table_1")
#                         primary_address_dt[temp] = v
#                 empty_fields = 0
#                 for i in range(len(table_1_fields)):
#                     if primary_address_dt.get(table_1_fields[i], "") == "":
#                         empty_fields += 1
#                 if len(table_1_fields) == empty_fields:
#                     file_name_sequence_counter -= 1
#                     continue
#                 temp_address = val[index]
#                 val.pop(index)

#                 barcode = "(" + tracking_ids_batch[index_p] + ")" if tracking_ids_batch else ""
#                 primary_address_dt["barcode"] = barcode
#                 tracking_id = tracking_ids_batch[index_p] if tracking_ids_batch else ""
#                 master_table_dict[primary_cell].append(primary_address_dt)
#                 if len(str(file_name_sequence_counter)) != len(str(total_length)):
#                     total_length_temp = str(total_length)
#                     file_name_sequence_counter_temp = str(file_name_sequence_counter)
#                     diff = abs(len(total_length_temp) - len(file_name_sequence_counter_temp))
#                     file_name_sequence_counter_temp = "0" * diff + file_name_sequence_counter_temp
#                 else:
#                     file_name_sequence_counter_temp = str(file_name_sequence_counter)

#                 kwargs["index"] = file_name_sequence_counter_temp
#                 kwargs["notice_data"]["master_table_dict"] = master_table_dict | duplicated_tables_dict
#                 kwargs["notice_id"] = notice_id
#                 notice_file_data = make_notice_doc_file(**kwargs)
#                 file_name_sequence_counter -= 1

#                 notice_id -= 1
#                 input_file, output_file, filename = (
#                     notice_file_data["input_file_path"],
#                     notice_file_data["output_file_path"],
#                     notice_file_data["filename"],
#                 )
#                 kwargs["input_file"] = input_file
#                 kwargs["output_file"] = output_file
#                 output_folder = local_pdf_directory_path

#                 if kwargs.get("enable_qrcode", False):
#                     generate_qrcode_doc(**kwargs)

#                 doc2pdf_converter(output_folder, **kwargs)
#                 make_even_pages(output_file, filename, local_pdf_directory_path)

#                 kwargs["tracking_ids"] += tracking_ids_batch
#                 # redis_instance.lpush(failed_tracking_ids_key, *tracking_ids_batch)
#                 # return res, notice_id_list
#                 kwargs["filenames"].append(output_file)
#                 filename = filename + ".pdf"
#                 kwargs["local_pdf_file_names"].append(filename)
#                 kwargs["primary_addresses"].append(json.dumps(primary_address_dt))

#                 if preview:
#                     return "success", notice_id_list, failed_tracking_id_list

#                 primary_address_text = primary_address_dt.get("table_1_address_text", "")
#                 primary_name = primary_address_dt.get("table_1_cell_1_name", "")
#                 primary_key = primary_name + primary_address_text

#                 if not address_file_map.get(primary_key):
#                     address_file_map[primary_key] = {
#                         "filename": filename,
#                         "primary_address": json.dumps(primary_address_dt),
#                         "tracking_id": tracking_id,
#                         "created": datetime.now(),
#                         "notice_id": kwargs["notice_id"],
#                     }
#                 else:
#                     address_file_map[primary_key] = {
#                         "filename": filename,
#                         "primary_address": json.dumps(primary_address_dt),
#                         "tracking_id": tracking_id,
#                         "created": datetime.now(),
#                         "notice_id": kwargs["notice_id"],
#                     }

#                 primary_address_dt.clear()
#                 val.insert(index, temp_address)
#                 master_table_dict[primary_cell].pop()
#                 index_p += 1

#         kwargs["address_file_map"] = address_file_map
#         kwargs["tracking_ids"] += tracking_ids_batch
#         return "success", notice_id_list, tracking_ids_batch
#     except Exception as e:
#         logger.error(f"generate_notice_for_all_addresses.exception: {str(e)}")
#         failed_tracking_id_list = tracking_ids_batch
#         return str(e), notice_id_list, failed_tracking_id_list


# def get_next_tracking_id(redis_key):
#     logger.info(f"notice.helpers.get_next_tracking_id")
#     redis_ = redis.Redis(host=REDIS["HOST"], port=REDIS["PORT"])
#     id = redis_.lpop(redis_key)
#     if id:
#         id = id.decode("utf-8")
#     else:
#         id = ""
#     logger.debug(f"get_next_tracking_id.id: {id}")
#     return id


# def make_even_pages(filepath, filename, local_pdf_directory_path):
#     logger.info(f"notice.helpers.make_even_pages")

#     logger.debug(f"{filepath}, {filename},{local_pdf_directory_path}")
#     pdf = PyPDF2.PdfFileReader(open(filepath, "rb"))
#     no_of_pages = pdf.getNumPages()
#     if no_of_pages % 2 != 0:
#         outPdf = PyPDF2.PdfFileWriter()
#         outPdf.appendPagesFromReader(pdf)
#         outPdf.addBlankPage()
#         temp_file = os.path.join(local_pdf_directory_path, f"temp_{filename}.pdf")
#         outPdf.write(open(temp_file, "wb"))
#         shutil.copyfile(temp_file, filepath)
#         os.remove(temp_file)
#     return


# def get_directory_and_file_paths(**kwargs):
#     volume_mount_directory = NOTICE_VOLUME_MOUNT_DIRECTORY
#     preview = kwargs.get("preview", False)

#     if preview:
#         local_batch_directory_path = os.path.join(volume_mount_directory, "previews", kwargs["batch_id"])
#     else:
#         local_batch_directory_path = os.path.join(volume_mount_directory, "batches", kwargs["batch_id"])

#     local_draft_file_path = os.path.join(local_batch_directory_path, f'{kwargs["draft_id"]}.docx')
#     local_doc_directory_path = os.path.join(local_batch_directory_path, "docs")
#     local_pdf_directory_path = os.path.join(local_batch_directory_path, "pdfs")
#     local_qr_code_directory_path = os.path.join(local_batch_directory_path, "qrcodes")
#     local_merged_pdf_directory_path = os.path.join(local_batch_directory_path, "merged_pdf")
#     local_merged_pdf_zip_file_path = os.path.join(local_batch_directory_path, "compressed_merged_lot")
#     return (
#         local_batch_directory_path,
#         local_draft_file_path,
#         local_doc_directory_path,
#         local_pdf_directory_path,
#         local_qr_code_directory_path,
#         local_merged_pdf_directory_path,
#         local_merged_pdf_zip_file_path,
#     )


# def generate_qrcode_doc(**kwargs):
#     logger.info(f"tasks.notice.generate_qrcode_doc.batch_id :: {kwargs['batch_id']}")
#     try:
#         input_doc_file_path = kwargs["input_file"]
#         doc_filename = input_doc_file_path.split("/")[-1].rstrip(".docx")
#         changed_dt = kwargs["notice_data"]["changed_dt"]
#         payment_link = changed_dt.get("payment_link", None)
#         local_qr_code_directory_path = kwargs["local_qr_code_directory_path"]

#         if payment_link:
#             qr_code_file_path = os.path.join(local_qr_code_directory_path, f"{doc_filename}.png")
#             doc_file = docx.Document(input_doc_file_path)
#             qr_code = pyqrcode.create(payment_link)
#             qr_code.png(qr_code_file_path, scale=5)
#             doc_file.add_picture(qr_code_file_path)
#             last_paragraph = doc_file.paragraphs[-1]
#             last_paragraph.alignment = WD_ALIGN_PARAGRAPH.CENTER
#             doc_file.save(input_doc_file_path)
#         else:
#             raise Exception("Missing payment_link")
#     except Exception as e:
#         raise Exception(f"generate_qrcode_doc.exception: {str(e)}")



"""
delegator.py
Usage: All delegator tasks
"""
import json
from datetime import datetime
from celery.canvas import chain
import requests
from ..settings import (
    LITIGATION_SERVICE_BASE_URL,
    SCRAPE_SERVICE_BASE_URL,
    NOTICE_SERVICE_BASE_URL,
    QUEUE_SERVICE_BASE_URL,
    COMMUNICATION_SERVICE_BASE_URL,
)
import logging
from celery import shared_task, group, chord
from .execution import optin, send_communications
from .report import calculate_report_data
from ..utils import _dict_key_filter

logger = logging.getLogger(__name__)


@shared_task(bind=True, name="chord_callback")
def chord_callback(self, *args, **kwargs):
    """
    Chord callback
    """
    print("Chord callback")
    return True


@shared_task(bind=True, name="execution_batch_number_callback")
def execution_batch_number_callback(self, *args, **kwargs):
    """
    Execution batch number callback
    """
    batch_number = kwargs["batch_number"]
    batch_id = kwargs["batch_id"]
    queue = kwargs["queue"]
    last_batch = kwargs["last_batch"]
    user = (kwargs["user"],)
    request_id = kwargs["request_id"]
    batch_status = {}
    success = []
    failed = []
    optin_partial_tasks = []
    total_batches = None
    execution_data_url = f"{QUEUE_SERVICE_BASE_URL}/execution_data?batch_number={batch_number}&batch_id={batch_id}"
    try:
        execution_data = requests.request(
            method="GET",
            url=execution_data_url,
            headers={
                "Content-Type": "application/json",
                "X-CG-User": json.dumps(user),
                "X-Request-ID": request_id,
            },
        )
        logger.info(
            f"execution_data.status_code_{batch_id} :: {execution_data.status_code}"
        )
    except Exception as e:
        logger.info(f"execution_data.exception_{batch_id} :: {str(e)}")
        self.retry(queue=queue, kwargs=kwargs, countdown=20, max_retries=5)

    if execution_data.status_code not in (200, 201):
        logger.info(f"execution_data.text_{batch_id} :: {execution_data.text}")
        self.retry(queue=queue, kwargs=kwargs, countdown=20, max_retries=5)

    execution_data = json.loads(execution_data.text)
    print(f"{execution_data} - execution_data")
    for data in execution_data["data"]:

        response = json.loads(data["response"])
        status_code = response["status_code"]
        loan_id = data["loan_id"]
        status = data["status"]
        total_batches = data["total_batches"]

        if status == "SUCCESS":
            success.append(loan_id)
        if status == "FAIL":
            failed.append(
                {
                    "reason": str(response["response"]),
                    "loan_id": loan_id,
                    "status_code": status_code,
                }
            )

    batch_status["failed"] = failed
    batch_status["success"] = success

    batch_status_update_url = f"{QUEUE_SERVICE_BASE_URL}/batch_status_update"

    try:
        batch_data = requests.request(
            method="POST",
            url=batch_status_update_url,
            data=json.dumps(
                {
                    "batch_id": batch_id,
                    "batch_number": batch_number,
                    "batch_status": batch_status,
                    "total_batches": total_batches,
                }
            ),
            headers={
                "Content-Type": "application/json",
                "X-CG-User": json.dumps(user),
                "X-Request-ID": request_id,
            },
        )
        logger.info(
            f"batch_status_update.status_code_{batch_id} :: {batch_data.status_code}"
        )

    except Exception as e:
        logger.info(f"batch_status_update.exception{batch_id} :: {str(e)}")
        self.retry(queue=queue, kwargs=kwargs, countdown=20, max_retries=5)

    if batch_data.status_code not in (200, 201):
        logger.info(f"batch_status_update.text_{batch_id} :: {batch_data.text}")
        self.retry(queue=queue, kwargs=kwargs, countdown=20, max_retries=5)

    batch_data = json.loads(batch_data.text)
    data = batch_data["data"][0]
    extras = json.loads(data["extras"])
    batch_number = data["batch_number"]

    type_of_task = data["type_of_task"]
    total_batches = data["total_batches"]
    company_id = data["company_id"]
    author = data["author"]
    total_loans = data["total_loans"]

    loan_ids = extras["loan_ids"]
    queue = extras["queue"]
    user = extras["user"]
    request_id = extras["request_id"]
    payload = extras["payload"]
    template_id = payload.get("template_id", None)
    if type_of_task in ("dtmf_ivr"):
        template_name = payload.get("template_name", None)
        rule_id = payload.get("rule_id", None)
        rule_name = payload.get("rule_name", None)
        applied_filter = payload.get("applied_filter", None)
    else:
        template_name = payload.get("comm_dict", {}).get("template_name", None)
        rule_id = payload.get("comm_dict", {}).get("rule_id", None)
        rule_name = payload.get("comm_dict", {}).get("rule_name", None)
        applied_filter = payload.get("comm_dict", {}).get("applied_filter", None)
    if last_batch:
        calculate_report_data_kwargs = {
            "company_id": company_id,
            "queue": queue,
            "author": author,
            "batch_id": batch_id,
            "type_of_task": type_of_task,
            "total_loans": total_loans,
            "template_name": template_name,
            "template_id": template_id,
            "rule_id": rule_id,
            "rule_name": rule_name,
            "applied_filter": applied_filter,
            "user": user,
            "request_id": request_id,
        }
        calculate_report_data.apply_async(
            queue=queue, kwargs=calculate_report_data_kwargs
        )
        return True

    if str(batch_number) == str(total_batches):
        last_batch = True
    if loan_ids not in ([], None):
        for loan_id in loan_ids:
            if type_of_task in (
                "email",
                "sms",
                "voice",
                "whatsapp",
                "call",
                "dtmf_ivr",
            ):
                communication_kwargs = {
                    "loan_id": loan_id,
                    "company_id": company_id,
                    "queue": queue,
                    "author": author,
                    "batch_id": batch_id,
                    "batch_number": batch_number,
                    "total_batches": total_batches,
                    "type_of_task": type_of_task,
                    "total_loans": total_loans,
                    "comm_type": payload["type_of_comm"],
                    "payload": payload,
                    "user": user,
                    "request_id": request_id,
                }
                optin_partial_tasks.append(
                    send_communications.s(**communication_kwargs)
                )
            else:
                optin_kwargs = {
                    "loan_id": loan_id,
                    "company_id": company_id,
                    "queue": queue,
                    "author": author,
                    "batch_id": batch_id,
                    "batch_number": batch_number,
                    "total_batches": total_batches,
                    "type_of_task": type_of_task,
                    "total_loans": total_loans,
                    "user": user,
                    "request_id": request_id,
                }
                optin_partial_tasks.append(optin.s(**optin_kwargs))
        chord(
            group(optin_partial_tasks),
            body=execution_batch_number_callback.s(
                queue=queue,
                batch_id=batch_id,
                batch_number=batch_number,
                last_batch=last_batch,
                user=user,
                request_id=request_id,
            ),
        ).apply_async(queue=queue)
    return True


@shared_task(bind=True, name="delegator", ignore_result=False)
def delegator(self, *args, **kwargs):
    """
    Delegator
    """
    batch_id = kwargs["batch_id"]
    url = (
        f"{QUEUE_SERVICE_BASE_URL}/get_batch_number?batch_number=1&batch_id={batch_id}"
    )
    try:
        batch_data = requests.request(
            method="GET",
            url=url,
            headers={
                "Content-Type": "application/json",
                "X-CG-User": json.dumps(kwargs["user"]),
                "X-Request-ID": kwargs["request_id"],
            },
        )
        logger.info(f"get_batch_number.status_code :: {batch_data.status_code}")
    except Exception as e:
        logger.info(f"get_batch_number.exception :: {str(e)}")
        self.retry(kwargs=kwargs, countdown=20, max_retries=5)

    if batch_data.status_code not in (200, 201):
        logger.info(f"get_batch_number.text :: {batch_data.text}")
        self.retry(kwargs=kwargs, countdown=20, max_retries=5)

    batch_data = json.loads(batch_data.text)

    data = batch_data["data"][0]
    extras = json.loads(data["extras"])
    batch_number = data["batch_number"]
    type_of_task = data["type_of_task"]
    total_batches = data["total_batches"]
    company_id = data["company_id"]
    author = data["author"]
    total_loans = data["total_loans"]

    loan_ids = extras["loan_ids"]
    queue = extras["queue"]
    user = extras["user"]
    request_id = extras["request_id"]
    payload = extras["payload"]

    if loan_ids not in ([], None):
        optin_partial_tasks = []
        for loan_id in loan_ids:
            if type_of_task in (
                "email",
                "sms",
                "voice",
                "whatsapp",
                "call",
                "whatsapp_bot",
                "dtmf_ivr",
            ):
                communication_kwargs = {
                    "loan_id": loan_id,
                    "company_id": company_id,
                    "queue": queue,
                    "author": author,
                    "batch_id": batch_id,
                    "batch_number": batch_number,
                    "total_batches": total_batches,
                    "type_of_task": type_of_task,
                    "total_loans": total_loans,
                    "comm_type": payload["type_of_comm"],
                    "payload": payload,
                    "user": user,
                    "request_id": request_id,
                }
                optin_partial_tasks.append(
                    send_communications.s(**communication_kwargs)
                )
            else:
                optin_kwargs = {
                    "loan_id": loan_id,
                    "company_id": company_id,
                    "queue": queue,
                    "author": author,
                    "batch_id": batch_id,
                    "batch_number": batch_number,
                    "total_batches": total_batches,
                    "type_of_task": type_of_task,
                    "total_loans": total_loans,
                    "user": user,
                    "request_id": request_id,
                }
                optin_partial_tasks.append(optin.s(**optin_kwargs))
    chord(
        group(optin_partial_tasks),
        body=execution_batch_number_callback.s(
            queue=queue,
            batch_id=batch_id,
            batch_number=batch_number,
            last_batch=True if total_batches == 1 else False,
            user=user,
            request_id=request_id,
        ),
    ).apply_async(queue=queue)
    return True



import json

from ..settings import USER_SERVICE_BASE_URL, API_USER_AUTHENTICATION_TOKEN
from ..choices import API_USER_DETAILS
import traceback
import requests as r

import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def get_users(company):
    logger.info(f"user_service.get_users")
    user = API_USER_DETAILS
    user["authentication_token"] = API_USER_AUTHENTICATION_TOKEN
    url = USER_SERVICE_BASE_URL.rstrip("/") + "/internal/company/users"
    headers = {
        "authenticationtoken": user.get("authentication_token"),
        "role": user["role"],
        "Content-Type": "application/json",
        "X-CG-User": json.dumps(user),
        "X-CG-Company": json.dumps(company),
    }

    req_payload = {
        "company_id": company["company_id"],
    }

    logger.info(f"user_service.get_users.url: {url}")
    logger.debug(f"user_service.get_users.req_payload: {req_payload}")

    try:
        response = r.request("POST", url=url, json=req_payload, headers=headers, timeout=300)
        logger.debug(f"user_service.get_users.req_status : {response.status_code}")
        if response.status_code != 200:
            logger.error(f"exception.user_service.get_users.req_err : {response.text}")
            raise Exception(response.text)
        data = response.json()
        data = data["data"]
        if data["users"]:
            return data["users"]
        else:
            raise Exception("no_user_data_found")
    except Exception as e:
        error_msg = f"exception.user_service.get_users : {e}"
        logger.error(
            error_msg,
            "".join(traceback.format_tb(e.__traceback__)),
        )
        raise Exception(error_msg)



"""
Utils.py
Usage: Serves Utility
"""
import boto3
import json
import requests
import logging
import shortuuid
import urllib
import re

from datetime import timedelta, datetime
from pathlib import Path
from functools import wraps
from quart import current_app as app, make_response, request
import uuid
from .choices import (
    ReminderOffsetTypeChoices,
    ErrorCodeMessages,
    CAMPAIGN_SUMMARY_CHOICES,
    COMPLETION_SUMMARY_CHOICES,
    COLOR_CODING,
    ERROR_FIELDS_COUNT,
    ERROR_DICT_MAPPING,
    PROD_ENV,
    ExportServiceCallType,
    MODULE,
    SOURCE,
)
from .settings import (
    AWS_ACCESS_KEY_ID,
    AWS_SECRET_ACCESS_KEY,
    AWS_DEFAULT_REGION,
    COMMUNICATION_SERVICE_BASE_URL,
    LITIGATION_SERVICE_BASE_URL,
    SENDGRID_API_KEY,
    USER_SERVICE_BASE_URL,
    ENV,
    REPORT_SENDGRID_TEMPLATE_ID,
)
from http import HTTPStatus
from phonenumbers.phonenumberutil import region_code_for_country_code
from pytz import country_timezones, timezone
from cg_utils import mask_data


logger = logging.getLogger(__name__)


class BadEnvException(Exception):
    def __init__(self, message):
        self.errorCode = 501
        self.errorMessage = message
        return


class VerifyEnv:
    def __init__(self):
        self.used_configs = [
            "LOG_LEVEL",
            "POSTGRES",
            "APP_NAME",
            "REDIS",
            "USER_SERVICE_BASE_URL",
        ]

    def verify(self):
        unverified_envs = []
        success = True
        for i in self.used_configs:
            env_var = app.config.get(i, None)
            if env_var is None or "":
                unverified_envs.append(i)
                success = False

        return success, unverified_envs


def is_nullish(data_to_check: tuple) -> bool:
    nullish = ("", None, "null")
    for data in data_to_check:
        if data in nullish:
            return True
    return False


def _dict_key_filter(d, keys):
    return {k: v for k, v in d.items() if k not in keys}


def _make_batch(iterable, n=1):
    count = 1
    l = len(iterable)
    for ndx in range(0, l, n):
        yield iterable[ndx : min(ndx + n, l)]


def api_payload_validator(schema):
    """
    return response for request payload
    {
        "message": "success",
        "output": "Address conversion assigned to queue successfully",
        "data":
    }, 200
    """

    def innerfunc(func):
        @wraps(func)
        async def innerfunc1(*args, **kwargs):
            request_data = await request.get_data()
            request_data = json.loads(request_data)
            validation_errors = schema().validate(request_data)
            if validation_errors:
                return await make_response(
                    json.dumps(
                        {
                            "message": "failed",
                            "output": "payload is not valid",
                            "data": validation_errors,
                        }
                    ),
                    400,
                )
            return await func(*args, **kwargs)

        return innerfunc1

    return innerfunc


#############################################################################################
######################## Notifications methods ##############################################
#############################################################################################


def datetime_offset(trigger_time: str, reminder_offset_type: str, reminder_offset: list, now_time):
    offset_resolved_list = []
    for offset in reminder_offset:
        if reminder_offset_type == ReminderOffsetTypeChoices.minutes.value:
            offset_time = trigger_time - timedelta(minutes=offset)
        elif reminder_offset_type == ReminderOffsetTypeChoices.hours.value:
            offset_time = trigger_time - timedelta(hours=offset)
        elif reminder_offset_type == ReminderOffsetTypeChoices.days.value:
            offset_time = trigger_time - timedelta(days=offset)
        print(f"trigger_time - {offset_time} now_time - {now_time}")
        if not offset_time < now_time:
            offset_resolved_list.append((offset_time - now_time).seconds)

    print(f"utils.datetime_offset :: {offset_resolved_list}")
    return offset_resolved_list


def create_directory_if_not_exists(directory: str):
    return Path(directory).mkdir(parents=True, exist_ok=False)


def delete_file_if_exists(file_path: str):
    return Path(file_path).unlink(missing_ok=True)


def get_s3_client():
    return boto3.client(
        "s3",
        aws_access_key_id=AWS_ACCESS_KEY_ID,
        aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
        region_name=AWS_DEFAULT_REGION,
        endpoint_url="https://s3."+AWS_DEFAULT_REGION+".amazonaws.com",
    )


def send_mail_api(payload: dict, token: str, role: str, request_id):
    base_url = COMMUNICATION_SERVICE_BASE_URL
    url = f"{base_url}/mail"

    headers = {
        "authenticationtoken": token,
        "role": role,
        "accept": "application/json",
        "Content-Type": "application/json",
        "X-Request-ID": str(request_id),
    }
    try:
        result = requests.request(
            method="POST",
            url=url,
            headers=headers,
            json=payload,
        )
        logger.info(f"send_mail_api.result.text: {result.text}")

        data = json.loads(result.text)
        status = data.get("status")
        logger.info(f"send_mail_api.result.status: {status}")
    except Exception as e:
        logger.info(f"send_mail_api.result.exception: {str(e)}")
        return False
    return True


def sendgrid_api(payload: dict):
    logger.info(f"sendgrid_api")
    unique_mail_id = shortuuid.ShortUUID().random(length=8)
    headers = {
        "authorization": f"Bearer {SENDGRID_API_KEY}",
        "content-type": "application/json",
        "Accept": "application/json",
    }
    try:
        mail_data = {
            "from": payload["from_data"],
            "subject": payload["subject"],
            "personalizations": [
                {
                    "to": [{"email": payload["to_email"]}],
                    "custom_args": {"unique_mail_id": unique_mail_id},
                }
            ],
            "content": [{"type": "text/html", "value": payload["email_body"]}],
            "reply_to": {"email": "reports@credgenics.com"},
        }
        response = requests.request(
            method="POST",
            url="https://api.sendgrid.com/v3/mail/send",
            data=json.dumps(mail_data),
            headers=headers,
        )
        logger.info(f"sendgrid_api.response {response}")
        if response.status_code != HTTPStatus.ACCEPTED.value:
            return False
    except Exception as e:
        logger.info(f"send_mail_api.result.exception: {str(e)}")
        return False
    return True


def format_api_response_text(response):
    logger.info(f"format_api_response")
    status_code = response.status_code
    res_text = response.text
    if res_text and status_code != 500:
        return str(res_text)
    elif status_code == HTTPStatus.INTERNAL_SERVER_ERROR.value:
        return ErrorCodeMessages.internal_server_error.value
    elif status_code == HTTPStatus.REQUEST_TIMEOUT.value:
        return ErrorCodeMessages.request_timeout.value
    elif status_code == HTTPStatus.BAD_GATEWAY.value:
        return ErrorCodeMessages.bad_gateway.value
    elif status_code == HTTPStatus.GATEWAY_TIMEOUT.value:
        return ErrorCodeMessages.gateway_timeout.value


def get_campaign_summary(
    campaign_export_output: dict,
    channel: str,
    failed_count=0,
    attempted_live=False,
    trigger_errors=[],
    completion_summary: bool = False,
) -> dict:
    """_summary_

    Args:
        campaign_export_output (dict): webhook details of a campaign
        channel (str): type of communication
        completion_summary (bool): true if the summary is to be created for campaign completion report

    Returns:
        dict: returns summary of a campaign
    """
    campaign_summary = {}
    campaign_export_errors = []
    if campaign_export_output.get("error_counts") and isinstance(campaign_export_output.get("error_counts"), list):
        campaign_export_errors = campaign_export_output["error_counts"]
    if trigger_errors:
        campaign_export_errors.extend(trigger_errors)
    triggered_count = campaign_export_output.get("triggered_count", 0)
    if attempted_live:
        campaign_summary["attempted"] = failed_count + triggered_count
    campaign_summary["triggered"] = triggered_count
    if completion_summary:
        campaign_summary_keys = COMPLETION_SUMMARY_CHOICES.get(channel, None)
    else:
        campaign_summary_keys = CAMPAIGN_SUMMARY_CHOICES.get(channel, None)
    if campaign_summary_keys:
        campaign_summary_keys = campaign_summary_keys.split(",")
        for key in campaign_summary_keys:
            campaign_summary[key] = campaign_export_output.get(f"{key}_count", 0)
    if not completion_summary:
        campaign_summary["error_counts"] = []
        if campaign_export_errors:

            sorted_error_items = sorted(
                campaign_export_errors,
                key=lambda item: item["count"],
                reverse=True,
            )
            sorted_error_items_length = len(sorted_error_items)

            campaign_summary["error_counts"] = get_pie_chart_formatted_data(sorted_error_items[:ERROR_FIELDS_COUNT])
            if sorted_error_items_length > ERROR_FIELDS_COUNT:
                others = {}
                others["id"] = "others"
                others["label"] = "others"
                others["value"] = sum(
                    list(
                        map(
                            lambda x: x["count"],
                            sorted_error_items[ERROR_FIELDS_COUNT:],
                        )
                    )
                )
                campaign_summary["error_counts"].append(others)
            campaign_summary["colors"] = COLOR_CODING

    return campaign_summary


def get_pie_chart_formatted_data(data):
    result = []
    for error in data:
        sub_data = {}
        for k1, k2 in ERROR_DICT_MAPPING.items():
            if k2 == "error_name":
                error[k2] = " ".join(list(error[k2].split("_")))
            sub_data[k1] = error[k2]
        result.append(sub_data)
    return result


def encode_param(data):
    encoded_data = urllib.parse.quote(data, safe=" ")
    return encoded_data


def list_chunker(data, chunk_size):
    """splits a list into chunks of size chunk_size

    Args:
        data(list): list to split
        chunk_size (int): chunk size
    Returns:
        data(list): list of chunks
    """
    return list(data[index : index + chunk_size] for index in range(0, len(data), chunk_size))


def create_query(
    table,
    set_clause,
    from_values_list,
    from_columns_list,
    from_alias_name,
    where,
    operation,
):
    """creates a single update query

    Args:
        table(string): name of table,
        set_clause(dict): columns to be updated,
        from_values_list(list): list of values to be updated,
        from_columns_list(list): columns to be used in update query
        from_alias_name(string): alias name for values to be updated,
        where(dict) : columns to be used in where condition,
        operation(string): operation to be performed
    Returns:
        query(string): update query

    Sample query:
        update table_name
        set column = nv.column
        from
            ( values
                (12, 'EURO'),
                (18, 'DOLLAR'),
                (13, 'Pound')
            ) as nv (id, column)
        where account.id = nv.id ;
    """
    query = ""
    set_clause_list = []
    where_list = []
    if operation.lower() == "update":
        for key, value in set_clause.items():
            set_clause_list.append(f"{key} = {from_alias_name}.{value}")
        for key, value in where.items():
            where_list.append(f"{table}.{key} = {from_alias_name}.{value}")
        if set_clause_list:
            set_clause = ",".join(set_clause_list)
            from_values = ",".join(from_values_list)
            where_clause = " and ".join(where_list)
            from_columns = ",".join(from_columns_list)
            query = f""" update {table} set {set_clause} from (values {from_values}) as {from_alias_name} ({from_columns}) where {where_clause};
            """
    return query


def get_users_details(company_id, user_ids, user, request_id, company_details):
    user_details = {"user_emails": [], "user_numbers": [], "profile_id": None}
    user_id_list = user_ids.split(",")
    user_emails = []
    user_numbers = []
    url = f"{USER_SERVICE_BASE_URL}/internal/company/users"
    payload = {
        "company_id": company_id,
        "user_ids": user_id_list,
        "include_chief_admin": True,
    }
    users = requests.request(
        method="POST",
        url=url,
        data=json.dumps(payload),
        headers={
            "Content-Type": "application/json",
            "X-CG-User": json.dumps(user),
            "X-Request-ID": request_id,
            "X-CG-Company": json.dumps(company_details),
        },
    )
    if users.status_code != HTTPStatus.OK.value:
        user_list = []
    else:
        try:
            user_list = json.loads(users.text)["data"]["users"]
        except Exception as e:
            user_list = []
    sms_mobile_data = {}
    if user_list:
        for user in user_list:
            if user["user_id"] == user_id_list[0]:
                user_details["author_profile_id"] = user["profile_id"]
            if user.get("email"):
                user_emails.append(user.get("email"))

            if user.get("country_isd_code") not in sms_mobile_data and user.get("mobile"):
                sms_mobile_data[user.get("country_isd_code")] = []

            if user.get("mobile"):
                sms_mobile_data[user.get("country_isd_code")].append(user.get("mobile"))

    user_details["user_emails"] = user_emails
    user_details["user_numbers"] = sms_mobile_data
    return user_details


def is_valid_uuid(value):
    try:
        uuid.UUID(str(value))

        return True
    except ValueError:
        return False


async def get_current_datetime(country_isd_code):
    country_isd_code = int(country_isd_code.replace("+", ""))
    country_code = region_code_for_country_code(country_isd_code)
    time_zone = country_timezones[country_code][0]
    current_datetime = datetime.now().astimezone(timezone(time_zone))
    return current_datetime


def validate_email(email: str):
    """
    verify belows list of emails
    mynam.dewe.ew@google.co.in
    tarun.kumar.1234@gmail.com
    tarun.kumar@credgenics.com
    dantala.satish@stfc.co.in
    anand.andy.139@gmail.com
    """
    pattern = "^[a-zA-Z0-9._-]+[@]\w+[-]?\w+([.]\w{2,20}){1,}$"
    if re.search(pattern, email):
        return True
    else:
        return False


def format_email_dict(email_list):
    return list(map(lambda email: {"email": email}, email_list))


def get_export_mail_data_helper(export_task, presigned_url, company_trademark, service=None):
    logger.info(f"get_export_mail_data_helper.export_task:{export_task}")
    export_metadata = export_task.get("data")
    if export_metadata and not isinstance(export_metadata, dict):
        export_metadata = json.loads(export_metadata)
    to = [{"email": export_task.get("author_email")}]
    cc = [{"email": "demo.agent@credgenics.com", "name": "Demo Agent"}]
    bcc = []
    logger.info(f"get_export_mail_data_helper.service:{service}")
    template_id = REPORT_SENDGRID_TEMPLATE_ID
    if service and service == ExportServiceCallType.DELIVERY_REPORT_CONSUMER.value:
        campaign_author_emails = export_metadata.get("campaign_author_emails", [])
        to = format_email_dict(campaign_author_emails)
        cc.extend(format_email_dict(export_metadata.get("cc_emails", [])))
        bcc = format_email_dict(export_metadata.get("bcc_emails", []))
        template_id = export_metadata.get("template_id")
    logger.debug(
        f"get_export_mail_data_helper.cc:{cc},get_export_mail_data_helper.bcc:{bcc},get_export_mail_data_helper.to:{to},get_export_mail_data_helper.template_id:{template_id}"
    )
    return get_mail_data(
        export_task=export_task,
        presigned_url=presigned_url,
        company_trademark=company_trademark,
        export_metadata=export_metadata,
        template_id=template_id,
        to=to,
        cc=cc,
        bcc=bcc,
    )


def get_mail_data(export_task, presigned_url, company_trademark, export_metadata, template_id, to, cc=None, bcc=None):
    mail_data = {
        "from": {
            "name": f"{export_task['export_category'].replace('_',' ').capitalize()}",
            "email": "reports@credgenics.com",
        },
        "personalizations": [
            {
                "to": to,
                "dynamic_template_data": {
                    "report_url": presigned_url,
                    "company": company_trademark,
                    "campaign_id": export_metadata.get("campaign_id"),
                    "channel": export_metadata.get("channel").replace("_", " ").capitalize(),
                    "status": export_metadata.get("delivery_status"),
                    "template_name": export_metadata.get("template_name"),
                    "campaign_name": export_metadata.get("name"),
                    "rule_name": export_metadata.get("rule_name"),
                    "filters": export_metadata.get("filters"),
                    "range": export_metadata.get("range"),
                    "campaign_link": export_metadata.get("campaign_link"),
                    "camapign_summary_keys": export_metadata.get("camapign_summary_keys"),
                    "camapign_summary_values": export_metadata.get("camapign_summary_values"),
                    "subject": export_metadata.get("subject"),
                    "warning": export_metadata.get("warning"),
                    "triggered_time": export_metadata.get("triggered_time").split(".")[0]
                    if export_metadata.get("triggered_time")
                    else "",
                    "env": ENV if str(ENV).lower() != PROD_ENV else None,
                },
            }
        ],
        "template_id": template_id,
        "source": SOURCE,
        "module": MODULE,
    }
    if cc:
        mail_data["personalizations"][0]["cc"] = cc
    if bcc:
        mail_data["personalizations"][0]["bcc"] = bcc
    return mail_data


async def execute_db_query(query, db_instance):
    app.logger.info(f"app.utils.execute_db_query")
    status_code = HTTPStatus.OK.value
    message = "Records fetched successfully"
    data = []
    try:
        app.logger.debug(f"app.utils.execute_db_query.query:: {query}")
        data = await db_instance.execute_raw_select_query(query)
        app.logger.debug(f"app.utils.execute_db_query.data :: {data}")

    except Exception as e:
        app.logger.error(f"app.utils.execute_db_query.exception {str(e)}")
        status_code = HTTPStatus.INTERNAL_SERVER_ERROR.value
        message = f"Failed to fetch campaign error records. {str(e)}"
        data = []

    if not isinstance(data, list) or isinstance(data, str):
        status_code = HTTPStatus.INTERNAL_SERVER_ERROR.value
        message = "Failed to fetch campaign error records"
        data = []

    return status_code, message, data


async def get_communication_masked_data(data, masked_variables, channel, applicant_type):
    app.logger.info(f"app.utils.get_communication_masked_data")
    if channel != "email":
        masking_variable = "contact_to"
        if applicant_type == "co_applicant":
            keys_alias = {"co_applicant_contact_number": masking_variable}
        elif applicant_type == "applicant":
            keys_alias = {
                "applicant_contact_number": masking_variable,
            }
        else:
            keys_alias = {
                "applicant_contact_number": masking_variable,
                "co_applicant_contact_number": masking_variable,
            }
    else:
        if applicant_type == "co_applicant":
            keys_alias = [
                {"co_applicant_email": "contact_to"},
            ]
        elif applicant_type == "applicant":
            keys_alias = [
                {"applicant_email": "contact_to"},
            ]
        else:
            keys_alias = [{"applicant_email": "contact_to", "co_applicant_email": "contact_to"}]

    if channel != "email":
        masked_variables_copy = masked_variables.copy()
        for key in keys_alias:
            if masked_variables_copy.get(key):
                masked_variables_copy[keys_alias[key]] = masked_variables_copy.pop(key)
        data = await mask_data(data, keys_to_be_masked=masked_variables_copy)
    else:
        for alias in keys_alias:
            masked_variables_copy = masked_variables.copy()
            for key in alias:
                if masked_variables_copy.get(key):
                    masked_variables_copy[alias[key]] = masked_variables_copy.pop(key)
            data = await mask_data(data, keys_to_be_masked=masked_variables_copy)
    return data


def get_excluded_columns(columns):
    excluded_columns = ""
    for column in columns:
        if excluded_columns:
            excluded_columns += ","
        excluded_columns += f"{column}=EXCLUDED.{column}"
    return excluded_columns


def generate_billing_report(user: dict, company: dict, batch_id: str, case_type: str):
    logger.info(f"utils.generate_billing_report.company_id : {company['company_id']}")
    headers = {
        "authenticationtoken": user["authentication_token"],
        "Content-type": "application/json",
        "X-CG-User": json.dumps(user),
    }
    url = f"{LITIGATION_SERVICE_BASE_URL.rstrip('/')}/billing-email"
    logger.info(f"utils.generate_billing_report.url : {url}")
    payload = {"company_id": company["company_id"], "batch_id": batch_id, "case_type": case_type}
    try:
        resp = requests.post(
            url=url,
            data=json.dumps({**payload, "triggered_from": "approval_request"}),
            headers=headers,
        )
        logger.info(f"utils.generate_billing_report.status_code : {resp.status_code}")
        if resp.status_code != HTTPStatus.OK.value:
            raise Exception(resp.text)
    except Exception as e:
        logger.error(f"utils.generate_billing_report.exception : {str(e)}")



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='my_quart_app',
    version='1.1.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    # entry_points={
    #     'console_scripts': [
    #         'my_quart_app = app.run_server:main',
    #     ]
    # },
)


# import requests
# import json
# from urllib.error import URLError
# from app.settings import RECOVERY_SERVICE_BASE_URL
# import logging

# logger = logging.getLogger(__name__)
# logger.setLevel(logging.DEBUG)


# def get_loan_data(**kwargs):
#     logger.info(f"tasks.notice.helpers.get_loan_data")
#     logger.info(f"tasks.notice.helpers.get_loan_data.batch_id :: {kwargs['batch_id']}")

#     url = RECOVERY_SERVICE_BASE_URL.rstrip("/") + f"/notice/loan/details"

#     headers = {
#         "authenticationtoken": kwargs.get("user").get("authentication_token"),
#         "role": kwargs.get("user").get("role"),
#         "Content-Type": "application/json",
#         "X-CG-User": json.dumps(kwargs.get("user", {})),
#         "X-Request-Id": kwargs.get("X-Request-Id", ""),
#         "X-CG-Company": json.dumps(kwargs.get("company", {})),
#     }
#     req_payload = {
#         "loan_ids": kwargs.get("loan_ids", ""),
#         "allocation_month": kwargs.get("allocation_month", ""),
#         "creditline": kwargs.get("creditline", ""),
#         "company_id": kwargs.get("company_id", ""),
#     }

#     logger.debug(f"tasks.notice.helpers.get_loan_data.url: {url}")
#     logger.debug(f"tasks.notice.helpers.get_loan_data.req_payload: {req_payload}")

#     try:
#         response = requests.post(url=url, data=json.dumps(req_payload), headers=headers, timeout=60)
#         logger.debug(f"get_loan_data.response.status_code : {response.status_code}")
#     except URLError as url_error:
#         try:
#             ResponseData = url_error.read().decode("utf8", "ignore")
#             logger.debug(f"get_loan_data.URLError.ResponseData: {str(ResponseData)}")
#         except Exception as exc:
#             logger.error(f"get_loan_data.URLError.ResponseData.exception: {str(exc)}")
#         logger.error(f"get_loan_data.URLError: {str(url_error)}")
#         raise Exception(f"get_loan_data.URLError: {str(url_error)}")
#     except TimeoutError as te:
#         logger.error(f"get_loan_data.TimeoutError: {str(te)}")
#         raise Exception(f"get_loan_data.TimeoutError: {str(te)}")
#     except ConnectionError as ce:
#         logger.error(f"get_loan_data.ConnectionError: {str(ce)}")
#         raise Exception(f"get_loan_data.ConnectionError: {str(ce)}")
#     except Exception as e:
#         logger.error(f"error name: {e.__class__.__name__}")
#         logger.error(f"error args: {e.args}")
#         logger.error(f"error: {str(e)}")
#         raise Exception(f"get_loan_data.general_exception: {str(e)}")

#     if response.status_code != 200:
#         logger.debug(f"loan_service.get_loan_data.err_response : {response.text}")
#         raise Exception(f"get_loan_data.err_response: {response.text}")

#     loan_data = json.loads(response.text)
#     return loan_data.get("output", {})


# def get_borrower_language(question_dict):
#     applicant_language = question_dict.get("applicant_language")
#     co_applicants = question_dict.get("co_applicant", [])
#     co_applicant_language = ""
#     for co_applicant in co_applicants:
#         co_applicant_language = co_applicant.get("co_applicant_language")
#     return {
#         "applicant_language": applicant_language,
#         "co_applicant_language": co_applicant_language,
#     }



# import shutil
# from celery import shared_task
# import redis
# from app.settings import (
#     NOTICE_VOLUME_MOUNT_DIRECTORY,
#     REDIS,
#     S3_BUCKET_ENDPOINT,
#     S3_BUCKET_NAME,
#     QUEUE_SERVICE_BASE_URL,
# )
# from app.tasks.notice.notice_callback.commons import (
#     delete_redis_keys,
#     generate_notices_zip,
#     generate_failed_notices_report_excel,
#     generate_tracking_excel,
#     generate_tracking_excel_with_merged_pdf_file_mapping,
#     send_batch_completion_email,
#     upload_file_to_s3,
#     generate_unused_tracking_ids_excel,
#     get_batch_notices,
#     create_merge_pdf_and_zip,
# )

# from app.utils import get_s3_client
# import os
# import logging
# import requests
# import json
# import asyncio

# logger = logging.getLogger(__name__)


# @shared_task(bind=True, name="generate_physical_notices_batch_callback")
# def generate_physical_notices_batch_callback(self, *args, **kwargs):

#     redis_instance = redis.Redis(host=REDIS["HOST"], port=REDIS["PORT"])
#     callback_kwargs = json.loads(redis_instance.get(f"callback_kwargs_{kwargs['batch_id']}").decode("utf-8"))
#     redis_instance.close()

#     if not callback_kwargs:
#         logger.error(
#             f"notice_callback.generate_physical_notice_batch_callback.error: callback_kwargs not found on redis"
#         )
#         return False

#     kwargs = {**kwargs, **callback_kwargs}

#     logger.info(f"notice_callback.physical_notice.generate_physical_notices_batch_callback.kwargs: {kwargs}")

#     message_flag = False
#     try:
#         kwargs["X-Request-Id"] = kwargs["request_id"]
#         s3 = get_s3_client()

#         volume_mount_directory = NOTICE_VOLUME_MOUNT_DIRECTORY
#         local_batch_directory_path = os.path.join(volume_mount_directory, "batches", kwargs["batch_id"])

#         local_draft_file_path = os.path.join(local_batch_directory_path, f'{kwargs["draft_id"]}.docx')
#         local_doc_directory_path = os.path.join(local_batch_directory_path, "docs")
#         local_pdf_directory_path = os.path.join(local_batch_directory_path, "pdfs")
#         local_zip_file_path_without_format = os.path.join(local_batch_directory_path, "notices")
#         local_zip_file_path = local_zip_file_path_without_format + ".zip"
#         local_tracking_excel_file_path = os.path.join(local_batch_directory_path, "tracking.xlsx")
#         local_batch_report_file_path = os.path.join(local_batch_directory_path, "report.xlsx")
#         local_tracking_ids_excel_path = os.path.join(local_batch_directory_path, "tracking_ids.xlsx")

#         local_merged_pdf_directory_path = kwargs["local_merged_pdf_directory_path"]
#         local_merged_pdf_zip_file_path = kwargs["local_merged_pdf_zip_file_path"] + ".zip"

#         s3_zip_file_path = f"physical_notice_zip/{kwargs['batch_id']}.zip"
#         s3_zip_file_link = f"{S3_BUCKET_ENDPOINT}/{s3_zip_file_path}"

#         s3_tracking_excel_path = f"physical_notice_reports/tracking/{kwargs['batch_id']}.xlsx"
#         s3_tracking_excel_link = f"{S3_BUCKET_ENDPOINT}/{s3_tracking_excel_path}"

#         # temp fix for tracking report : with file name mapping
#         local_tracking_excel_file_with_merge_file_name_mapping_path = os.path.join(
#             local_batch_directory_path, "tracking_with_merge_file_name_mapping.xlsx"
#         )
#         s3_tracking_excel_with_merge_file_name_mapping_path = (
#             f"physical_notice_reports/tracking/{kwargs['batch_id']}_with_merge_file_name_mapping.xlsx"
#         )
#         s3_tracking_excel_with_merge_file_name_mapping_link = (
#             f"{S3_BUCKET_ENDPOINT}/{s3_tracking_excel_with_merge_file_name_mapping_path}"
#         )
#         kwargs[
#             "local_tracking_excel_file_with_merge_file_name_mapping_path"
#         ] = local_tracking_excel_file_with_merge_file_name_mapping_path
#         kwargs[
#             "s3_tracking_excel_with_merge_file_name_mapping_path"
#         ] = s3_tracking_excel_with_merge_file_name_mapping_path
#         kwargs[
#             "s3_tracking_excel_with_merge_file_name_mapping_link"
#         ] = s3_tracking_excel_with_merge_file_name_mapping_link

#         s3_report_batch_excel_path = f"physical_notice_reports/reports/{kwargs['batch_id']}.xlsx"
#         s3_report_batch_excel_link = f"{S3_BUCKET_ENDPOINT}/{s3_report_batch_excel_path}"
#         s3_tracking_ids_excel_path = f"physical_notice_reports/reports/{kwargs['batch_id']}_tracking_ids.xlsx"
#         s3_tracking_ids_excel_link = f"{S3_BUCKET_ENDPOINT}/{s3_tracking_ids_excel_path}"
#         batch_name = kwargs.get("batch_name")
#         batch_id = kwargs.get("batch_id")

#         kwargs["local_pdf_directory_path"] = local_pdf_directory_path
#         kwargs["local_zip_file_path"] = local_zip_file_path
#         kwargs["s3_zip_file_path"] = s3_zip_file_path
#         kwargs["s3_zip_file_link"] = s3_zip_file_link
#         kwargs["s3_tracking_excel_path"] = s3_tracking_excel_path
#         kwargs["s3_tracking_excel_link"] = s3_tracking_excel_link
#         kwargs["s3_report_batch_excel_path"] = s3_report_batch_excel_path
#         kwargs["s3_report_batch_excel_link"] = s3_report_batch_excel_link
#         kwargs["s3_tracking_ids_excel_path"] = s3_tracking_ids_excel_path
#         kwargs["s3_tracking_ids_excel_link"] = s3_tracking_ids_excel_link

#         notice_data = get_batch_notices(**kwargs)
#         if isinstance(notice_data, list):
#             notice_data = sorted(notice_data, key=lambda x: int(x["local_pdf_file_name"].split(".")[0]))
#         else:
#             logger.error(
#                 "notice_callback.physical_notice.generate_physical_notices_batch_callback || batch notices data couldn't be retrived"
#             )
#             raise Exception(
#                 "notice_callback.physical_notice.generate_physical_notices_batch_callback || batch notices data couldn't be retrived"
#             )

#         kwargs["notice_data"] = notice_data
#         merged_lot_size = kwargs.get("merged_lot_size", None)
#         notices_zip_generated = False
#         merging_process_success = False
#         count_of_notices = len(notice_data)

#         if batch_name:
#             merged_lot_zip_name = f"{batch_name}_{batch_id}_{count_of_notices}"
#         else:
#             merged_lot_zip_name = f"{batch_id}_{count_of_notices}"

#         s3_merged_pdf_zip_file_path = f"physical_notice_reports/merged_files/{merged_lot_zip_name}"
#         s3_merged_pdf_zip_file_link = f"{S3_BUCKET_ENDPOINT}/{s3_merged_pdf_zip_file_path}" + ".zip"
#         kwargs["s3_merged_pdf_zip_file_path"] = s3_merged_pdf_zip_file_path
#         kwargs["s3_merged_pdf_zip_file_link"] = s3_merged_pdf_zip_file_link

#         if len(notice_data):
#             if merged_lot_size:
#                 notice_id_merged_file_name_mapping = asyncio.run(create_merge_pdf_and_zip(**kwargs))
#                 logger.debug(
#                     "app.tasks.notice.notice_callback.physical_notice.generate_physical_notices_batch_callback || MERGE_NOTICE DONE"
#                 )
#                 if isinstance(notice_id_merged_file_name_mapping, bool):
#                     logger.error("couldn't create merge lot")
#                     merging_process_success = False
#                 else:
#                     logger.debug(
#                         f"notice_callback.notice_id_merged_file_name_mapping: {notice_id_merged_file_name_mapping}"
#                     )
#                     kwargs["notice_id_merged_file_name_mapping"] = notice_id_merged_file_name_mapping
#                     merging_process_success = True
#             notices_zip_generated = generate_notices_zip(
#                 local_pdf_directory_path=local_pdf_directory_path,
#                 local_zip_file_path=local_zip_file_path_without_format,
#             )
#             if notices_zip_generated:
#                 upload_file_to_s3(s3, local_zip_file_path, S3_BUCKET_NAME, s3_zip_file_path)
#                 kwargs["notices_zip_generated"] = True

#         if (notices_zip_generated) or (merged_lot_size and merging_process_success):
#             kwargs["file_path"] = local_tracking_excel_file_path
#             generate_tracking_excel(**kwargs)
#             upload_file_to_s3(
#                 s3,
#                 local_tracking_excel_file_path,
#                 S3_BUCKET_NAME,
#                 s3_tracking_excel_path,
#             )

#         kwargs["merging_process_success"] = merging_process_success

#         if merging_process_success:
#             generate_tracking_excel_with_merged_pdf_file_mapping(**kwargs)
#             upload_file_to_s3(
#                 s3,
#                 local_tracking_excel_file_with_merge_file_name_mapping_path,
#                 S3_BUCKET_NAME,
#                 s3_tracking_excel_with_merge_file_name_mapping_path,
#             )

#         # generate unused tracking ids excel
#         kwargs["local_tracking_ids_excel_path"] = local_tracking_ids_excel_path
#         local_tracking_ids_excel_file_generated_flag = generate_unused_tracking_ids_excel(**kwargs)
#         if local_tracking_ids_excel_file_generated_flag:
#             upload_file_to_s3(
#                 s3,
#                 local_tracking_ids_excel_path,
#                 S3_BUCKET_NAME,
#                 s3_tracking_ids_excel_path,
#             )
#         kwargs["local_tracking_ids_excel_file_generated_flag"] = local_tracking_ids_excel_file_generated_flag

#         kwargs["local_batch_report_file_path"] = local_batch_report_file_path
#         batch_report_data = generate_failed_notices_report_excel(**kwargs)
#         kwargs["batch_report_data"] = batch_report_data
#         if os.path.exists(local_batch_report_file_path):
#             upload_file_to_s3(
#                 s3,
#                 local_batch_report_file_path,
#                 S3_BUCKET_NAME,
#                 s3_report_batch_excel_path,
#             )
#         send_batch_completion_email(**kwargs)

#         url = f"{QUEUE_SERVICE_BASE_URL}/update_batch_response"
#         data = {
#             "batch_id": kwargs["batch_id"],
#             "s3_error_report_excel_path": s3_report_batch_excel_path,
#             "s3_batch_report_excel_path": s3_tracking_excel_path,
#         }
#         headers = {
#             "Content-Type": "application/json",
#             "X-CG-User": json.dumps(kwargs["user"]),
#             "X-Request-ID": kwargs["request_id"],
#             "X-CG-Company": json.dumps(kwargs["company"]),
#         }

#         update_response = requests.request(method="PATCH", url=url, json=data, headers=headers)
#         logger.debug(
#             "physical_notice.generate_physical_notices_batch_callback.update_batch_operations url: %s,status_code: %s",
#             url,
#             update_response.status_code,
#         )

#         if update_response.status_code != 200:
#             logger.error(
#                 "physical_notice.generate_physical_notices_batch_callback.update_batch_operations response: %s",
#                 update_response.text,
#             )

#         message_flag = True
#     except Exception as e:
#         logger.error(f"generate_physical_notices_batch_callback.global_exception : {str(e)}")
#     finally:
#         delete_redis_keys(kwargs["batch_id"])
#         return message_flag


# @shared_task(bind=True, name="generate_physical_notice_preview_callback")
# def generate_physical_notice_preview_callback(self, *args, **kwargs):
#     logger.info("generate_physical_notice_preview_callback")
#     logger.debug(f"generate_physical_notice_preview_callback.kwargs: {kwargs}")

#     volume_mount_directory = NOTICE_VOLUME_MOUNT_DIRECTORY
#     local_batch_directory_path = os.path.join(volume_mount_directory, "previews", kwargs["batch_id"])

#     shutil.rmtree(local_batch_directory_path, ignore_errors=True)
#     logger.debug(f"generate_physical_notice_preview_callback.dir_removal.info :{local_batch_directory_path} deleted")
#     delete_redis_keys(kwargs["batch_id"])

#     return True



# import shutil
# from celery import shared_task
# from app.settings import NOTICE_VOLUME_MOUNT_DIRECTORY
# import os
# import logging

# from app.tasks.notice.notice_callback.commons import delete_redis_keys

# logger = logging.getLogger(__name__)
# logger.setLevel(logging.DEBUG)


# @shared_task(bind=True, name="generate_digital_notice_preview_callback")
# def generate_digital_notice_preview_callback(self, *args, **kwargs):
#     logger.info("generate_digital_notice_preview_callback")
#     logger.debug(f"generate_digital_notice_preview_callback.kwargs: {kwargs}")

#     volume_mount_directory = NOTICE_VOLUME_MOUNT_DIRECTORY
#     local_batch_directory_path = os.path.join(volume_mount_directory, "previews", kwargs["batch_id"])

#     shutil.rmtree(local_batch_directory_path, ignore_errors=True)
#     logger.debug(f"generate_digital_notice_preview_callback.dir_removal.info :{local_batch_directory_path} deleted")
#     delete_redis_keys(kwargs["batch_id"])

#     return True



# import logging

# logger = logging.getLogger(__name__)
# logger.setLevel(logging.DEBUG)


# @shared_task(bind=True, name="assign_generate_notice_preview_queue")
# def assign_generate_notice_preview_queue(self, *args, **kwargs):
#     logger.info("assign_generate_notice_preview_queue")
#     redis_instance = redis.Redis(host=REDIS["HOST"], port=REDIS["PORT"])

#     logger.debug(f"assign_generate_notice_preview_queue.kwargs: {kwargs}")

#     queue = kwargs["queue"]
#     mode = kwargs["mode"]
#     batch_id = kwargs["batch_id"]

#     if mode == "digital":
#         type_of_task = "digital"
#     else:
#         type_of_task = "physical"
#         initial_notice_id_redis_key = f"initial_notice_id_redis_key_{batch_id}"
#         redis_instance.setex(initial_notice_id_redis_key, 24 * 60 * 60, json.dumps(1001))

#     if kwargs.get("linked_loans"):
#         linked_loan = kwargs["linked_loans"][0]
#         loan_ids = linked_loan.get("loan_ids", [])
#     else:
#         loan_ids = kwargs["loan_ids"]

#     physical_notice_trackings_ids_redis_key = ""
#     tasks = []

#     task_kwargs = {
#         "company_id": kwargs["company_id"],
#         "draft_id": kwargs["draft_id"],
#         "draft_details": kwargs["draft_details"],
#         "loan_ids": loan_ids,
#         "allocation_month": kwargs["allocation_month"],
#         "author": kwargs["author"],
#         "token": kwargs["token"],
#         "mode": kwargs["mode"],
#         "role": kwargs["role"],
#         "queue": queue,
#         "batch_id": batch_id,
#         "batch_number": kwargs["batch_number"],
#         "total_batches": 1,
#         "type_of_task": type_of_task,
#         "total_loans": 1,
#         "document_type": kwargs["document_type"],
#         "physical_notice_trackings_ids_redis_key": physical_notice_trackings_ids_redis_key,
#         "X-Request-Id": kwargs["X-Request-Id"],
#         "s3_link_uuid": kwargs["s3_link_uuid"],
#         "preview": True,
#         "user": kwargs["user"],
#         "linked_loan": linked_loan if kwargs.get("linked_loans") else {},
#         "is_linked_loan": kwargs["is_linked_loan"],
#         "request_id": kwargs["request_id"],
#         "local_batch_directory_path": kwargs["local_batch_directory_path"],
#         "local_draft_file_path": kwargs["local_draft_file_path"],
#         "local_doc_directory_path": kwargs["local_doc_directory_path"],
#         "local_pdf_directory_path": kwargs["local_pdf_directory_path"],
#         "local_pdf_directory_path": kwargs["local_pdf_directory_path"],
#         "local_qr_code_directory_path": kwargs["local_qr_code_directory_path"],
#         "local_merged_pdf_directory_path": kwargs["local_merged_pdf_directory_path"],
#         "local_merged_pdf_zip_file_path": kwargs["local_merged_pdf_zip_file_path"],
#         "BUCKET_NAME": kwargs["BUCKET_NAME"],
#         "draft_fields_redis_key": kwargs["draft_fields_redis_key"],
#         "company": kwargs["company"],
#         "initial_notice_id_redis_key": initial_notice_id_redis_key if mode == "physical" else None,
#         "enable_qrcode": kwargs["enable_qrcode"],
#     }

#     callback_kwargs = {
#         "queue": kwargs["queue"],
#         "batch_id": batch_id,
#         "batch_number": kwargs["batch_number"],
#         "company_id": kwargs["company_id"],
#         "token": kwargs["token"],
#         "role": kwargs["role"],
#         "draft_id": kwargs["draft_id"],
#         "type_of_task": type_of_task,
#         "author": kwargs["author"],
#         "user": kwargs["user"],
#         "request_id ": kwargs["X-Request-Id"],
#         "company": kwargs["company"],
#     }

#     task_kwargs = _dict_key_filter(task_kwargs, ["loan_ids", "linked_loan"])
#     redis_instance.setex(f"notice_kwargs_{batch_id}", 24 * 60 * 60, json.dumps(task_kwargs))
#     redis_instance.setex(f"callback_kwargs_{batch_id}", 24 * 60 * 60, json.dumps(callback_kwargs))
#     redis_instance.close()

#     if mode == "digital":
#         if kwargs["is_linked_loan"]:
#             tasks.append(handle_generate_digital_notice.s(**{"batch_id": batch_id, "linked_loan": linked_loan}))
#             chord(
#                 group(tasks),
#                 body=generate_digital_notice_preview_callback.s(batch_id=batch_id),
#             ).apply_async(queue=queue)
#         else:
#             tasks.append(handle_generate_digital_notice.s(**{"batch_id": batch_id, "loan_ids": loan_ids}))
#             chord(
#                 group(tasks),
#                 body=generate_digital_notice_preview_callback.s(batch_id=batch_id),
#             ).apply_async(queue=queue)

#     else:
#         if kwargs["is_linked_loan"]:
#             tasks.append(generate_physical_notices.s(**{"batch_id": batch_id, "linked_loan": linked_loan}))
#             chord(
#                 group(tasks),
#                 body=generate_physical_notice_preview_callback.s(batch_id=batch_id),
#             ).apply_async(queue=queue)
#         else:
#             tasks.append(generate_physical_notices.s(**{"batch_id": batch_id, "loan_ids": loan_ids}))
#             chord(
#                 group(tasks),
#                 body=generate_physical_notice_preview_callback.s(batch_id=batch_id),
#             ).apply_async(queue=queue)

#     return True
